=== modified file 'Makefile'
--- Makefile	2008-10-28 18:00:20 +0000
+++ Makefile	2009-01-10 03:26:15 +0000
@@ -9,13 +9,14 @@
 #DEBUG=-ggdb3
 # For info about _FORTIFY_SOURCE, see
 # <http://gcc.gnu.org/ml/gcc-patches/2004-09/msg02055.html>
-FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIE -pie
-LINK_FORTIFY=-z relro -pie
+FORTIFY=-D_FORTIFY_SOURCE=2 -fstack-protector-all -fPIC -fPIE
+LINK_FORTIFY_LD=-z relro -fPIE
+LINK_FORTIFY=-pie
 #COVERAGE=--coverage
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
 htmldir=man
-version=1.0.2
+version=1.0.3
 SED=sed
 
 ## Use these settings for a traditional /usr/local install
@@ -45,7 +46,7 @@
 CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
 	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
 	-DVERSION='"$(version)"'
-LDFLAGS=$(COVERAGE) $(LINK_FORTIFY)
+LDFLAGS=$(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=cd $(dir $<); xsltproc --nonet --xinclude \
@@ -76,7 +77,7 @@
 PLUGINS=plugins.d/password-prompt plugins.d/mandos-client \
 	plugins.d/usplash plugins.d/splashy plugins.d/askpass-fifo
 CPROGS=plugin-runner $(PLUGINS)
-PROGS=mandos mandos-keygen $(CPROGS)
+PROGS=mandos mandos-keygen mandos-list $(CPROGS)
 DOCS=mandos.8 plugin-runner.8mandos mandos-keygen.8 \
 	plugins.d/mandos-client.8mandos \
 	plugins.d/password-prompt.8mandos mandos.conf.5 \
@@ -163,6 +164,11 @@
 		--expression='s/^\(VERSION="\)[^"]*"$$/\1$(version)"/' \
 		$@
 
+mandos-list: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(version = "\)[^"]*"$$/\1$(version)"/' \
+		$@
+
 mandos.lsm: Makefile
 	$(SED) --in-place \
 		--expression='s/^\(Version:\).*/\1\t$(version)/' \
@@ -170,6 +176,9 @@
 	$(SED) --in-place \
 		--expression='s/^\(Entered-date:\).*/\1\t$(shell date --rfc-3339=date --reference=Makefile)/' \
 		$@
+	$(SED) --in-place \
+		--expression='s/\(mandos_\)[0-9.]\+\(\.orig\.tar\.gz\)/\1$(version)\2/' \
+		$@
 
 plugins.d/mandos-client: plugins.d/mandos-client.o
 	$(LINK.o) $(GNUTLS_LIBS) $(AVAHI_LIBS) $(GPGME_LIBS) \
@@ -194,7 +203,8 @@
 run-client: all keydir/seckey.txt keydir/pubkey.txt
 	./plugin-runner --plugin-dir=plugins.d \
 		--config-file=plugin-runner.conf \
-		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt
+		--options-for=mandos-client:--seckey=keydir/seckey.txt,--pubkey=keydir/pubkey.txt \
+		$(CLIENTARGS)
 
 # Used by run-client
 keydir/seckey.txt keydir/pubkey.txt: mandos-keygen
@@ -203,7 +213,7 @@
 
 # Run the server with a local config
 run-server: confdir/mandos.conf confdir/clients.conf
-	./mandos --debug --configdir=confdir
+	./mandos --debug --configdir=confdir $(SERVERARGS)
 
 # Used by run-server
 confdir/mandos.conf: mandos.conf

=== modified file 'NEWS'
--- NEWS	2008-10-17 18:56:25 +0000
+++ NEWS	2009-01-06 02:42:53 +0000
@@ -1,6 +1,18 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
+Version 1.0.3 (2009-01-06)
+* Server
+** Now tries to change to user and group "_mandos" before falling back
+   to trying the old values "mandos", "nobody:nogroup", and "65534".
+** Now does not abort on startup even if no clients are defined in
+   clients.conf.
+
+* Client
+** Plugins named "*.dpkg-bak" are now ignored.
+** Hopefully fixed compilation failure on some architectures where the
+   C compiler does not recognize the "-z" option as a linker option.
+
 Version 1.0.2 (2008-10-17)
 * mandos-keygen now signs the encrypted key blobs.  This signature is
   not currently verified by mandos-client, but this may change in the

=== modified file 'README'
--- README	2008-10-28 18:00:20 +0000
+++ README	2009-01-12 22:02:33 +0000
@@ -152,16 +152,14 @@
     cryptsetup, this plugin listens to the same FIFO as askpass would
     do.
   
-  (None of these take any options or reads any files.)
-  
   More plugins could easily be written and added by the system
   administrator; see the section called "WRITING PLUGINS" in
   plugin-runner(8mandos) to learn the plugin requirements.
 
 * Copyright
 
-    Copyright © 2008 Teddy Hogeborn
-    Copyright © 2008 Björn Påhlsson
+    Copyright © 2008,2009 Teddy Hogeborn
+    Copyright © 2008,2009 Björn Påhlsson
   
 ** License:
    

=== modified file 'TODO'
--- TODO	2008-11-09 06:40:29 +0000
+++ TODO	2009-01-12 22:02:33 +0000
@@ -6,6 +6,9 @@
    klogctl(6, NULL, 0); klogctl(7, NULL, 0);
 ** TODO [#C] IPv4 support
 
+* plugin-runner
+** TODO [#B] use scandir(3) instead of readdir(3)
+
 * mandos (server)
 ** TODO [#B] Log level				:bugs:
 ** TODO /etc/mandos/clients.d/*.conf
@@ -15,17 +18,11 @@
    Probably using D-Bus
    See also [[*Mandos-tools]]
 *** Client class
-    + getHostname
-    + getChecker
-    + setChecker
 *** Main server
-    + Clients
-      out_signature="ao"
-      Does this have to be "getClients" so as not to collide with the
-      interface name?
-    + setLogLevel
+    + SetLogLevel
       syslogger.setLevel(logging.WARNING)
-    + quit
+    + Quit
+    + [[http://log.ometer.com/2007-05.html][Best D-Bus practices]]
 ** TODO Implement --foreground			:bugs:
    [[info:standards:Option%20Table][Table of Long Options]]
 ** TODO Implement --socket
@@ -33,14 +30,23 @@
 ** TODO Date+time on console log messages	:bugs:
    Is this the default?
 ** TODO delete hook when clients fall out by timeout
-
-* Mandos-tools/utilities
-  All of this probably using D-Bus
-** TODO List clients
+   This will not be strictly necessary when the D-Bus interface is
+   implemented.
+
+* mandos.xml
+** [[file:mandos.xml::XXX][Document D-Bus interface]]
+
+* Provide and install /etc/dbus-1/system.d/mandos.conf
+
+* mandos-list
+*** Handle "no D-Bus server" and/or "no Mandos server found" better
+*** [#B] --dump option
 ** TODO Disable client
 ** TODO Enable client
 ** TODO Reset timer
 
+* Curses interface
+
 * mandos-keygen
 ** TODO "--secfile" option
    Using the "secfile" option instead of "secret"
@@ -49,11 +55,14 @@
 
 * [#A] Package
 ** /usr/share/initramfs-tools/hooks/mandos
-*** TODO Do not install in initrd.img if configured not to.
+*** TODO [#C] Do not install in initrd.img if configured not to.
     Use "/etc/initramfs-tools/conf.d/mandos"?  Definitely a debconf
     question.
-** TODO /etc/bash_completion.d/mandos
+** TODO [#C] /etc/bash_completion.d/mandos
    From XML sources directly?
+** TODO initramfs-tools-script						  :test:
+   Do not insert plugin-runner as keyscript if a kernel parameter
+   "mandos=off" is passed.
 
 
 #+STARTUP: showall

=== modified file 'clients.conf'
--- clients.conf	2008-09-21 13:42:34 +0000
+++ clients.conf	2009-01-08 03:54:06 +0000
@@ -14,7 +14,7 @@
 ;interval = 5m
 
 # What command to run as "the checker".
-;checker = fping -q -- %(host)s
+;checker = fping -q -- %%(host)s
 
 
 ;####

=== modified file 'debian/changelog'
--- debian/changelog	2008-10-17 18:56:25 +0000
+++ debian/changelog	2009-01-06 22:43:19 +0000
@@ -1,3 +1,26 @@
+mandos (1.0.3-2) unstable; urgency=low
+
+  * Removed some now-unused debconf files.
+  * Changed postinst scripts to not source debconf/confmodule.
+  * Removed po-debconf from build-depends.
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 06 Jan 2009 21:28:20 +0100
+
+mandos (1.0.3-1) unstable; urgency=low
+
+  * New upstream release.
+  * Add -Xlinker to linker flags to fix FTBFS for some architectures.
+    Thanks to Thiemo Seufer <ths@networkno.de> for the report and
+    fix. (Closes:  #509398)
+  * Remove debconf use altogether, thereby stopping debconf abuse.  Thanks
+    to Christian Perrier <bubulle@debian.org>. (Closes:  #509653)
+  * Add NEWS file to /usr/share/doc directories.
+  * Use and create "_mandos" user+group.  Rename old user+group created by
+    older versions of this package.
+  * Fix manual pages by adding build-depend on "docbook-xml".
+
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>  Tue, 06 Jan 2009 01:21:20 +0100
+
 mandos (1.0.2-1) unstable; urgency=low
 
   * New upstream release.

=== modified file 'debian/control'
--- debian/control	2008-09-30 03:19:39 +0000
+++ debian/control	2009-01-06 20:39:35 +0000
@@ -4,8 +4,8 @@
 Maintainer: Mandos Maintainers <mandos@fukt.bsnet.se>
 Uploaders: Teddy Hogeborn <teddy@fukt.bsnet.se>,
 	   Björn Påhlsson <belorn@fukt.bsnet.se>
-Build-Depends: debhelper (>= 7), docbook-xsl, libavahi-core-dev,
-	libgpgme11-dev, libgnutls-dev, xsltproc, po-debconf,
+Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
+	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
 	pkg-config
 Standards-Version: 3.8.0
 Vcs-Bzr: http://ftp.fukt.bsnet.se/pub/mandos/trunk

=== modified file 'debian/copyright'
--- debian/copyright	2008-10-15 19:54:11 +0000
+++ debian/copyright	2009-01-04 21:54:55 +0000
@@ -5,8 +5,8 @@
 Upstream-Source: <http://www.fukt.bsnet.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008 Teddy Hogeborn
-Copyright: Copyright © 2008 Björn Påhlsson
+Copyright: Copyright © 2008,2009 Teddy Hogeborn
+Copyright: Copyright © 2008,2009 Björn Påhlsson
 License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as

=== removed file 'debian/mandos-client.config'
--- debian/mandos-client.config	2008-09-21 04:22:50 +0000
+++ debian/mandos-client.config	1970-01-01 00:00:00 +0000
@@ -1,27 +0,0 @@
-#! /bin/sh
-#
-# config	Mandos Debconf configuration.
-#
-
-# Source debconf library.
-. /usr/share/debconf/confmodule
-if ! db_version 2.0; then
-    echo "mandos.config: need DebConf 2.0 or later"
-    exit 1
-fi
-
-set -e
-umask 022
-
-# Now, interaction.  Batch it in case any front ends can use this.
-db_beginblock
-
-# If this is a first time install then prompt 
-if [ "$1" = "configure" -a "$2" != "" ]; then
-	db_input high mandos-client/not-yet-configured || true
-fi
-
-db_endblock
-db_go || true
-
-exit 0

=== modified file 'debian/mandos-client.postinst'
--- debian/mandos-client.postinst	2008-09-26 04:54:35 +0000
+++ debian/mandos-client.postinst	2009-01-06 05:08:37 +0000
@@ -15,8 +15,6 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
 # Update the initial RAM file system image
 update_initramfs()
 {
@@ -27,10 +25,19 @@
 
 # Add user and group
 add_mandos_user(){
-    if ! getent passwd mandos >/dev/null; then
-	adduser --disabled-password  --quiet --system \
-	    --home /nonexistent --no-create-home \
-	    --gecos "Mandos password system" --group mandos
+    # Rename old "mandos" user and group
+    case "$(getent passwd mandos)" in
+	*:Mandos\ password\ system,,,:/nonexistent:/bin/false)
+	    usermod --login _mandos mandos
+	    groupmod --new-name _mandos mandos
+	    return
+	    ;;
+    esac
+    # Create new user and group
+    if ! getent passwd _mandos >/dev/null; then
+	adduser --system --force-badname --quiet --home /nonexistent \
+	    --no-create-home --group --disabled-password \
+	    --gecos "Mandos password system" _mandos
     fi
 }
 

=== removed file 'debian/mandos-client.templates'
--- debian/mandos-client.templates	2008-09-21 04:22:50 +0000
+++ debian/mandos-client.templates	1970-01-01 00:00:00 +0000
@@ -1,8 +0,0 @@
-Template: mandos-client/not-yet-configured
-Type: note
-_Description: Your system needs more configuration[ mandos-client]
- Your system can not function as a Mandos client until a
- password for this client has been added to the
- configuration on the Mandos server.  Please read
- /usr/share/doc/mandos-client/README.Debian.gz to find out
- how.

=== modified file 'debian/mandos.README.Debian'
--- debian/mandos.README.Debian	2008-09-21 04:22:50 +0000
+++ debian/mandos.README.Debian	2009-01-04 22:15:01 +0000
@@ -1,7 +1,7 @@
-The Mandos server cannot run without at least one configured client in
+The Mandos server is useless without at least one configured client in
 /etc/mandos/clients.conf.  To create one, install the "mandos-client"
 package on a client computer, and run "mandos-keygen --password" there
 to get a config file stanza.  Append that to /etc/mandos/clients.conf
 on the Mandos server.
 
- -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Sat, 20 Sep 2008 21:21:19 +0200
+ -- Teddy Hogeborn <teddy@fukt.bsnet.se>, Sun,  4 Jan 2009 22:59:22 +0100

=== removed file 'debian/mandos.config'
--- debian/mandos.config	2008-09-21 04:22:50 +0000
+++ debian/mandos.config	1970-01-01 00:00:00 +0000
@@ -1,27 +0,0 @@
-#! /bin/sh
-#
-# config	Mandos Debconf configuration.
-#
-
-# Source debconf library.
-. /usr/share/debconf/confmodule
-if ! db_version 2.0; then
-    echo "mandos.config: need DebConf 2.0 or later"
-    exit 1
-fi
-
-set -e
-umask 022
-
-# Now, interaction.  Batch it in case any front ends can use this.
-db_beginblock
-
-# If this is a first time install then prompt 
-if [ "$1" = "configure" -a "$2" != "" ]; then
-	db_input high mandos/not-yet-configured || true
-fi
-
-db_endblock
-db_go || true
-
-exit 0

=== modified file 'debian/mandos.postinst'
--- debian/mandos.postinst	2008-09-26 04:54:35 +0000
+++ debian/mandos.postinst	2009-01-06 05:08:37 +0000
@@ -15,14 +15,21 @@
 # If prerm fails during replacement due to conflict:
 #       <postinst> abort-remove in-favour <new-package> <version>
 
-. /usr/share/debconf/confmodule
-
 case "$1" in
     configure)
-	if ! getent passwd mandos >/dev/null; then
-	    adduser --disabled-password  --quiet --system \
-		--home /nonexistent --no-create-home \
-		--gecos "Mandos password system" --group mandos
+	# Rename old "mandos" user and group
+	case "$(getent passwd mandos)" in
+	    *:Mandos\ password\ system,,,:/nonexistent:/bin/false)
+		usermod --login _mandos mandos
+		groupmod --new-name _mandos mandos
+		;;
+	esac
+	# Create new user and group
+	if ! getent passwd _mandos >/dev/null; then
+	    adduser --system --force-badname --quiet \
+		--home /nonexistent --no-create-home --group \
+		--disabled-password --gecos "Mandos password system" \
+		_mandos
 	fi
 	;;
 

=== removed file 'debian/mandos.templates'
--- debian/mandos.templates	2008-09-21 04:22:50 +0000
+++ debian/mandos.templates	1970-01-01 00:00:00 +0000
@@ -1,9 +0,0 @@
-Template: mandos/not-yet-configured
-Type: note
-_Description: Your system needs more configuration[ mandos]
- Your system has not yet been completely configured as a
- Mandos server - clients need to be added to to
- /etc/mandos/clients.conf.  Please read
- /usr/share/doc/mandos/README.Debian.gz to find out how.
- .
- (The server has not been started.)

=== removed file 'debian/po/POTFILES.in'
--- debian/po/POTFILES.in	2008-09-21 04:22:50 +0000
+++ debian/po/POTFILES.in	1970-01-01 00:00:00 +0000
@@ -1,2 +0,0 @@
-[type: gettext/rfc822deb] mandos.templates
-[type: gettext/rfc822deb] mandos-client.templates

=== removed file 'debian/po/sv.po'
--- debian/po/sv.po	2008-09-21 04:22:50 +0000
+++ debian/po/sv.po	1970-01-01 00:00:00 +0000
@@ -1,66 +0,0 @@
-# SOME DESCRIPTIVE TITLE.
-# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
-# This file is distributed under the same license as the mandos package.
-# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
-#
-msgid ""
-msgstr ""
-"Project-Id-Version: 1.0\n"
-"Report-Msgid-Bugs-To: mandos@packages.debian.org\n"
-"POT-Creation-Date: 2008-09-20 23:01+0200\n"
-"PO-Revision-Date: 2008-09-21 06:01+0200\n"
-"Last-Translator: Teddy Hogeborn <teddy@fukt.bsnet.se>\n"
-"Language-Team: Swedish <tp-sv@listor.tp-sv.se>\n"
-"MIME-Version: 1.0\n"
-"Content-Type: text/plain; charset=UTF-8\n"
-"Content-Transfer-Encoding: 8bit\n"
-
-#. Type: note
-#. Description
-#: ../mandos.templates:1001
-msgid "Your system needs more configuration[ mandos]"
-msgstr "Ditt system behöver ytterligare konfigurering"
-
-#. Type: note
-#. Description
-#: ../mandos.templates:1001
-#| msgid ""
-#| "Your system has not yet been completely configured as a Mandos server - "
-#| "you need to setup /etc/mandos/clients.conf. Please read /usr/share/doc/"
-#| "mandos/README.Debian.gz to find out how."
-msgid ""
-"Your system has not yet been completely configured as a Mandos server - "
-"clients need to be added to to /etc/mandos/clients.conf.  Please read /usr/"
-"share/doc/mandos/README.Debian.gz to find out how."
-msgstr ""
-"Ditt system är inte helt inställd som en Mandos-server än -\n"
-"det behövs läggas till klienter i Mandos-serverns\n"
-"inställingar.  Var vänlig läs\n"
-"/usr/share/doc/mandos-client/README.Debian.gz för att få\n"
-"veta hur."
-
-#. Type: note
-#. Description
-#: ../mandos.templates:1001
-msgid "(The server has not been started.)"
-msgstr "(Servern har inte startats.)"
-
-#. Type: note
-#. Description
-#: ../mandos-client.templates:1001
-msgid "Your system needs more configuration[ mandos-client]"
-msgstr "Ditt system behöver ytterligare konfigurering"
-
-#. Type: note
-#. Description
-#: ../mandos-client.templates:1001
-msgid ""
-"Your system can not function as a Mandos client until a password for this "
-"client has been added to the configuration on the Mandos server.  Please "
-"read /usr/share/doc/mandos-client/README.Debian.gz to find out how."
-msgstr ""
-"Ditt system kan inte fungera som en Mandos-klient förrän\n"
-"ett krypterat lösenord har lagts till i Mandos-serverns\n"
-"inställingar.  Var vänlig läs\n"
-"/usr/share/doc/mandos-client/README.Debian.gz för att få\n"
-"veta hur."

=== modified file 'debian/rules'
--- debian/rules	2008-09-30 18:59:44 +0000
+++ debian/rules	2009-01-04 22:26:07 +0000
@@ -38,7 +38,6 @@
 	rm -f build-arch-stamp build-indep-stamp configure-stamp
 	dh_auto_clean
 	dh_clean
-	debconf-updatepo
 
 install: install-indep install-arch
 install-indep:
@@ -66,7 +65,6 @@
 	dh_testroot
 	dh_installchangelogs
 	dh_installdocs
-	dh_installdebconf
 	dh_link
 	dh_strip
 	dh_compress

=== modified file 'initramfs-tools-hook'
--- initramfs-tools-hook	2008-11-01 02:26:00 +0000
+++ initramfs-tools-hook	2008-12-10 01:26:02 +0000
@@ -51,11 +51,13 @@
     exit 1
 fi
 
-mandos_user="`{ getent passwd mandos \
+mandos_user="`{ getent passwd _mandos \
+		|| getent passwd mandos \
 		|| getent passwd nobody \
 		|| echo ::65534::::; } \
 	| awk --field-separator=: '{ print $3 }'`" 
-mandos_group="`{ getent group mandos \
+mandos_group="`{ getent group _mandos \
+		|| getent group mandos \
 		|| getent group nogroup \
 		|| echo ::65534:; } \
 	| awk --field-separator=: '{ print $3 }'`"

=== modified file 'mandos'
--- mandos	2008-11-09 06:40:29 +0000
+++ mandos	2009-01-08 03:54:06 +0000
@@ -11,7 +11,8 @@
 # and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2008 Teddy Hogeborn & Björn Påhlsson
+# Copyright © 2008,2009 Teddy Hogeborn
+# Copyright © 2008,2009 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,14 +66,14 @@
 import ctypes
 import ctypes.util
 
-version = "1.0.2"
+version = "1.0.3"
 
 logger = logging.Logger('mandos')
-syslogger = logging.handlers.SysLogHandler\
-            (facility = logging.handlers.SysLogHandler.LOG_DAEMON,
-             address = "/dev/log")
-syslogger.setFormatter(logging.Formatter\
-                        ('Mandos: %(levelname)s: %(message)s'))
+syslogger = (logging.handlers.SysLogHandler
+             (facility = logging.handlers.SysLogHandler.LOG_DAEMON,
+              address = "/dev/log"))
+syslogger.setFormatter(logging.Formatter
+                       ('Mandos: %(levelname)s: %(message)s'))
 logger.addHandler(syslogger)
 
 console = logging.StreamHandler()
@@ -81,11 +82,11 @@
 logger.addHandler(console)
 
 class AvahiError(Exception):
-    def __init__(self, value):
+    def __init__(self, value, *args, **kwargs):
         self.value = value
-        super(AvahiError, self).__init__()
-    def __str__(self):
-        return repr(self.value)
+        super(AvahiError, self).__init__(value, *args, **kwargs)
+    def __unicode__(self):
+        return unicode(repr(self.value))
 
 class AvahiServiceError(AvahiError):
     pass
@@ -111,16 +112,13 @@
                   a sensible number of times
     """
     def __init__(self, interface = avahi.IF_UNSPEC, name = None,
-                 servicetype = None, port = None, TXT = None, domain = "",
-                 host = "", max_renames = 32768):
+                 servicetype = None, port = None, TXT = None,
+                 domain = "", host = "", max_renames = 32768):
         self.interface = interface
         self.name = name
         self.type = servicetype
         self.port = port
-        if TXT is None:
-            self.TXT = []
-        else:
-            self.TXT = TXT
+        self.TXT = TXT if TXT is not None else []
         self.domain = domain
         self.host = host
         self.rename_count = 0
@@ -131,13 +129,13 @@
             logger.critical(u"No suitable Zeroconf service name found"
                             u" after %i retries, exiting.",
                             self.rename_count)
-            raise AvahiServiceError("Too many renames")
+            raise AvahiServiceError(u"Too many renames")
         self.name = server.GetAlternativeServiceName(self.name)
         logger.info(u"Changing Zeroconf service name to %r ...",
                     str(self.name))
-        syslogger.setFormatter(logging.Formatter\
+        syslogger.setFormatter(logging.Formatter
                                ('Mandos (%s): %%(levelname)s:'
-                               ' %%(message)s' % self.name))
+                                ' %%(message)s' % self.name))
         self.remove()
         self.add()
         self.rename_count += 1
@@ -149,10 +147,10 @@
         """Derived from the Avahi example code"""
         global group
         if group is None:
-            group = dbus.Interface\
-                    (bus.get_object(avahi.DBUS_NAME,
+            group = dbus.Interface(bus.get_object
+                                   (avahi.DBUS_NAME,
                                     server.EntryGroupNew()),
-                     avahi.DBUS_INTERFACE_ENTRY_GROUP)
+                                   avahi.DBUS_INTERFACE_ENTRY_GROUP)
             group.connect_to_signal('StateChanged',
                                     entry_group_state_changed)
         logger.debug(u"Adding Zeroconf service '%s' of type '%s' ...",
@@ -172,229 +170,198 @@
 # End of Avahi example code
 
 
+def _datetime_to_dbus(dt, variant_level=0):
+    """Convert a UTC datetime.datetime() to a D-Bus type."""
+    return dbus.String(dt.isoformat(), variant_level=variant_level)
+
+
 class Client(dbus.service.Object):
     """A representation of a client host served by this server.
     Attributes:
-    name:      string; from the config file, used in log messages
+    name:       string; from the config file, used in log messages
     fingerprint: string (40 or 32 hexadecimal digits); used to
                  uniquely identify the client
-    secret:    bytestring; sent verbatim (over TLS) to client
-    host:      string; available for use by the checker command
-    created:   datetime.datetime(); object creation, not client host
-    started:   bool()
-    last_checked_ok: datetime.datetime() or None if not yet checked OK
-    timeout:   datetime.timedelta(); How long from last_checked_ok
-                                     until this client is invalid
-    interval:  datetime.timedelta(); How often to start a new checker
-    stop_hook: If set, called by stop() as stop_hook(self)
-    checker:   subprocess.Popen(); a running checker process used
-                                   to see if the client lives.
-                                   'None' if no process is running.
+    secret:     bytestring; sent verbatim (over TLS) to client
+    host:       string; available for use by the checker command
+    created:    datetime.datetime(); (UTC) object creation
+    last_enabled: datetime.datetime(); (UTC)
+    enabled:    bool()
+    last_checked_ok: datetime.datetime(); (UTC) or None
+    timeout:    datetime.timedelta(); How long from last_checked_ok
+                                      until this client is invalid
+    interval:   datetime.timedelta(); How often to start a new checker
+    disable_hook:  If set, called by disable() as disable_hook(self)
+    checker:    subprocess.Popen(); a running checker process used
+                                    to see if the client lives.
+                                    'None' if no process is running.
     checker_initiator_tag: a gobject event source tag, or None
-    stop_initiator_tag:    - '' -
+    disable_initiator_tag:    - '' -
     checker_callback_tag:  - '' -
     checker_command: string; External command which is run to check if
                      client lives.  %() expansions are done at
                      runtime with vars(self) as dict, so that for
                      instance %(name)s can be used in the command.
-    Private attibutes:
-    _timeout: Real variable for 'timeout'
-    _interval: Real variable for 'interval'
-    _timeout_milliseconds: Used when calling gobject.timeout_add()
-    _interval_milliseconds: - '' -
+    use_dbus: bool(); Whether to provide D-Bus interface and signals
+    dbus_object_path: dbus.ObjectPath ; only set if self.use_dbus
     """
-    interface = u"org.mandos_system.Mandos.Clients"
-    
-    @dbus.service.method(interface, out_signature="s")
-    def getName(self):
-        "D-Bus getter method"
-        return self.name
-    
-    @dbus.service.method(interface, out_signature="s")
-    def getFingerprint(self):
-        "D-Bus getter method"
-        return self.fingerprint
-    
-    @dbus.service.method(interface, in_signature="ay",
-                         byte_arrays=True)
-    def setSecret(self, secret):
-        "D-Bus setter method"
-        self.secret = secret
-    
-    def _set_timeout(self, timeout):
-        "Setter function for the 'timeout' attribute"
-        self._timeout = timeout
-        self._timeout_milliseconds = ((self.timeout.days
-                                       * 24 * 60 * 60 * 1000)
-                                      + (self.timeout.seconds * 1000)
-                                      + (self.timeout.microseconds
-                                         // 1000))
-        # Emit D-Bus signal
-        self.TimeoutChanged(self._timeout_milliseconds)
-    timeout = property(lambda self: self._timeout, _set_timeout)
-    del _set_timeout
-    
-    @dbus.service.method(interface, out_signature="t")
-    def getTimeout(self):
-        "D-Bus getter method"
-        return self._timeout_milliseconds
-    
-    @dbus.service.signal(interface, signature="t")
-    def TimeoutChanged(self, t):
-        "D-Bus signal"
-        pass
-    
-    def _set_interval(self, interval):
-        "Setter function for the 'interval' attribute"
-        self._interval = interval
-        self._interval_milliseconds = ((self.interval.days
-                                        * 24 * 60 * 60 * 1000)
-                                       + (self.interval.seconds
-                                          * 1000)
-                                       + (self.interval.microseconds
-                                          // 1000))
-        # Emit D-Bus signal
-        self.IntervalChanged(self._interval_milliseconds)
-    interval = property(lambda self: self._interval, _set_interval)
-    del _set_interval
-    
-    @dbus.service.method(interface, out_signature="t")
-    def getInterval(self):
-        "D-Bus getter method"
-        return self._interval_milliseconds
-    
-    @dbus.service.signal(interface, signature="t")
-    def IntervalChanged(self, t):
-        "D-Bus signal"
-        pass
-    
-    def __init__(self, name = None, stop_hook=None, config=None):
+    def timeout_milliseconds(self):
+        "Return the 'timeout' attribute in milliseconds"
+        return ((self.timeout.days * 24 * 60 * 60 * 1000)
+                + (self.timeout.seconds * 1000)
+                + (self.timeout.microseconds // 1000))
+    
+    def interval_milliseconds(self):
+        "Return the 'interval' attribute in milliseconds"
+        return ((self.interval.days * 24 * 60 * 60 * 1000)
+                + (self.interval.seconds * 1000)
+                + (self.interval.microseconds // 1000))
+    
+    def __init__(self, name = None, disable_hook=None, config=None,
+                 use_dbus=True):
         """Note: the 'checker' key in 'config' sets the
         'checker_command' attribute and *not* the 'checker'
         attribute."""
-        dbus.service.Object.__init__(self, bus,
-                                     "/Mandos/Clients/%s"
-                                     % name.replace(".", "_"))
+        self.name = name
         if config is None:
             config = {}
-        self.name = name
         logger.debug(u"Creating client %r", self.name)
+        self.use_dbus = use_dbus
+        if self.use_dbus:
+            self.dbus_object_path = (dbus.ObjectPath
+                                     ("/Mandos/clients/"
+                                      + self.name.replace(".", "_")))
+            dbus.service.Object.__init__(self, bus,
+                                         self.dbus_object_path)
         # Uppercase and remove spaces from fingerprint for later
         # comparison purposes with return value from the fingerprint()
         # function
-        self.fingerprint = config["fingerprint"].upper()\
-                           .replace(u" ", u"")
+        self.fingerprint = (config["fingerprint"].upper()
+                            .replace(u" ", u""))
         logger.debug(u"  Fingerprint: %s", self.fingerprint)
         if "secret" in config:
             self.secret = config["secret"].decode(u"base64")
         elif "secfile" in config:
             with closing(open(os.path.expanduser
                               (os.path.expandvars
-                               (config["secfile"])))) \
-                               as secfile:
+                               (config["secfile"])))) as secfile:
                 self.secret = secfile.read()
         else:
             raise TypeError(u"No secret or secfile for client %s"
                             % self.name)
         self.host = config.get("host", "")
-        self.created = datetime.datetime.now()
-        self.started = False
+        self.created = datetime.datetime.utcnow()
+        self.enabled = False
+        self.last_enabled = None
         self.last_checked_ok = None
         self.timeout = string_to_delta(config["timeout"])
         self.interval = string_to_delta(config["interval"])
-        self.stop_hook = stop_hook
+        self.disable_hook = disable_hook
         self.checker = None
         self.checker_initiator_tag = None
-        self.stop_initiator_tag = None
+        self.disable_initiator_tag = None
         self.checker_callback_tag = None
-        self.check_command = config["checker"]
+        self.checker_command = config["checker"]
     
-    def start(self):
+    def enable(self):
         """Start this client's checker and timeout hooks"""
-        self.started = True
+        self.last_enabled = datetime.datetime.utcnow()
         # Schedule a new checker to be started an 'interval' from now,
         # and every interval from then on.
-        self.checker_initiator_tag = gobject.timeout_add\
-                                     (self._interval_milliseconds,
-                                      self.start_checker)
+        self.checker_initiator_tag = (gobject.timeout_add
+                                      (self.interval_milliseconds(),
+                                       self.start_checker))
         # Also start a new checker *right now*.
         self.start_checker()
-        # Schedule a stop() when 'timeout' has passed
-        self.stop_initiator_tag = gobject.timeout_add\
-                                  (self._timeout_milliseconds,
-                                   self.stop)
-        # Emit D-Bus signal
-        self.StateChanged(True)
-    
-    @dbus.service.signal(interface, signature="b")
-    def StateChanged(self, started):
-        "D-Bus signal"
-        pass
-    
-    def stop(self):
-        """Stop this client."""
-        if getattr(self, "started", False):
-            logger.info(u"Stopping client %s", self.name)
-        else:
+        # Schedule a disable() when 'timeout' has passed
+        self.disable_initiator_tag = (gobject.timeout_add
+                                   (self.timeout_milliseconds(),
+                                    self.disable))
+        self.enabled = True
+        if self.use_dbus:
+            # Emit D-Bus signals
+            self.PropertyChanged(dbus.String(u"enabled"),
+                                 dbus.Boolean(True, variant_level=1))
+            self.PropertyChanged(dbus.String(u"last_enabled"),
+                                 (_datetime_to_dbus(self.last_enabled,
+                                                    variant_level=1)))
+    
+    def disable(self):
+        """Disable this client."""
+        if not getattr(self, "enabled", False):
             return False
-        if getattr(self, "stop_initiator_tag", False):
-            gobject.source_remove(self.stop_initiator_tag)
-            self.stop_initiator_tag = None
+        logger.info(u"Disabling client %s", self.name)
+        if getattr(self, "disable_initiator_tag", False):
+            gobject.source_remove(self.disable_initiator_tag)
+            self.disable_initiator_tag = None
         if getattr(self, "checker_initiator_tag", False):
             gobject.source_remove(self.checker_initiator_tag)
             self.checker_initiator_tag = None
         self.stop_checker()
-        if self.stop_hook:
-            self.stop_hook(self)
-        # Emit D-Bus signal
-        self.StateChanged(False)
+        if self.disable_hook:
+            self.disable_hook(self)
+        self.enabled = False
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(dbus.String(u"enabled"),
+                                 dbus.Boolean(False, variant_level=1))
         # Do not run this again if called by a gobject.timeout_add
         return False
-    # D-Bus variant
-    Stop = dbus.service.method(interface)(stop)
     
     def __del__(self):
-        self.stop_hook = None
-        self.stop()
+        self.disable_hook = None
+        self.disable()
     
-    def checker_callback(self, pid, condition):
+    def checker_callback(self, pid, condition, command):
         """The checker has completed, so take appropriate actions."""
         self.checker_callback_tag = None
         self.checker = None
-        if os.WIFEXITED(condition) \
-               and (os.WEXITSTATUS(condition) == 0):
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(dbus.String(u"checker_running"),
+                                 dbus.Boolean(False, variant_level=1))
+        if (os.WIFEXITED(condition)
+            and (os.WEXITSTATUS(condition) == 0)):
             logger.info(u"Checker for %(name)s succeeded",
                         vars(self))
-            # Emit D-Bus signal
-            self.CheckerCompleted(True)
+            if self.use_dbus:
+                # Emit D-Bus signal
+                self.CheckerCompleted(dbus.Boolean(True),
+                                      dbus.UInt16(condition),
+                                      dbus.String(command))
             self.bump_timeout()
         elif not os.WIFEXITED(condition):
             logger.warning(u"Checker for %(name)s crashed?",
                            vars(self))
-            # Emit D-Bus signal
-            self.CheckerCompleted(False)
+            if self.use_dbus:
+                # Emit D-Bus signal
+                self.CheckerCompleted(dbus.Boolean(False),
+                                      dbus.UInt16(condition),
+                                      dbus.String(command))
         else:
             logger.info(u"Checker for %(name)s failed",
                         vars(self))
-            # Emit D-Bus signal
-            self.CheckerCompleted(False)
-    
-    @dbus.service.signal(interface, signature="b")
-    def CheckerCompleted(self, success):
-        "D-Bus signal"
-        pass
+            if self.use_dbus:
+                # Emit D-Bus signal
+                self.CheckerCompleted(dbus.Boolean(False),
+                                      dbus.UInt16(condition),
+                                      dbus.String(command))
     
     def bump_timeout(self):
         """Bump up the timeout for this client.
         This should only be called when the client has been seen,
         alive and well.
         """
-        self.last_checked_ok = datetime.datetime.now()
-        gobject.source_remove(self.stop_initiator_tag)
-        self.stop_initiator_tag = gobject.timeout_add\
-            (self._timeout_milliseconds, self.stop)
-    # D-Bus variant
-    bumpTimeout = dbus.service.method(interface)(bump_timeout)
+        self.last_checked_ok = datetime.datetime.utcnow()
+        gobject.source_remove(self.disable_initiator_tag)
+        self.disable_initiator_tag = (gobject.timeout_add
+                                      (self.timeout_milliseconds(),
+                                       self.disable))
+        if self.use_dbus:
+            # Emit D-Bus signal
+            self.PropertyChanged(
+                dbus.String(u"last_checked_ok"),
+                (_datetime_to_dbus(self.last_checked_ok,
+                                   variant_level=1)))
     
     def start_checker(self):
         """Start a new checker subprocess if one is not running.
@@ -410,18 +377,18 @@
         # is as it should be.
         if self.checker is None:
             try:
-                # In case check_command has exactly one % operator
-                command = self.check_command % self.host
+                # In case checker_command has exactly one % operator
+                command = self.checker_command % self.host
             except TypeError:
                 # Escape attributes for the shell
                 escaped_attrs = dict((key, re.escape(str(val)))
                                      for key, val in
                                      vars(self).iteritems())
                 try:
-                    command = self.check_command % escaped_attrs
+                    command = self.checker_command % escaped_attrs
                 except TypeError, error:
                     logger.error(u'Could not format string "%s":'
-                                 u' %s', self.check_command, error)
+                                 u' %s', self.checker_command, error)
                     return True # Try again later
             try:
                 logger.info(u"Starting checker %r for %s",
@@ -433,26 +400,22 @@
                 self.checker = subprocess.Popen(command,
                                                 close_fds=True,
                                                 shell=True, cwd="/")
-                self.checker_callback_tag = gobject.child_watch_add\
-                                            (self.checker.pid,
-                                             self.checker_callback)
-                # Emit D-Bus signal
-                self.CheckerStarted(command)
+                if self.use_dbus:
+                    # Emit D-Bus signal
+                    self.CheckerStarted(command)
+                    self.PropertyChanged(
+                        dbus.String("checker_running"),
+                        dbus.Boolean(True, variant_level=1))
+                self.checker_callback_tag = (gobject.child_watch_add
+                                             (self.checker.pid,
+                                              self.checker_callback,
+                                              data=command))
             except OSError, error:
                 logger.error(u"Failed to start subprocess: %s",
                              error)
         # Re-run this periodically if run by gobject.timeout_add
         return True
     
-    @dbus.service.signal(interface, signature="s")
-    def CheckerStarted(self, command):
-        pass
-    
-    @dbus.service.method(interface, out_signature="b")
-    def checkerIsRunning(self):
-        "D-Bus getter method"
-        return self.checker is not None
-    
     def stop_checker(self):
         """Force the checker process, if any, to stop."""
         if self.checker_callback_tag:
@@ -470,36 +433,168 @@
             if error.errno != errno.ESRCH: # No such process
                 raise
         self.checker = None
-    # D-Bus variant
-    StopChecker = dbus.service.method(interface)(stop_checker)
+        if self.use_dbus:
+            self.PropertyChanged(dbus.String(u"checker_running"),
+                                 dbus.Boolean(False, variant_level=1))
     
     def still_valid(self):
         """Has the timeout not yet passed for this client?"""
-        if not self.started:
+        if not getattr(self, "enabled", False):
             return False
-        now = datetime.datetime.now()
+        now = datetime.datetime.utcnow()
         if self.last_checked_ok is None:
             return now < (self.created + self.timeout)
         else:
             return now < (self.last_checked_ok + self.timeout)
-    # D-Bus variant
-    stillValid = dbus.service.method(interface, out_signature="b")\
-        (still_valid)
-    
-    del interface
+    
+    ## D-Bus methods & signals
+    _interface = u"org.mandos_system.Mandos.Client"
+    
+    # BumpTimeout - method
+    BumpTimeout = dbus.service.method(_interface)(bump_timeout)
+    BumpTimeout.__name__ = "BumpTimeout"
+    
+    # CheckerCompleted - signal
+    @dbus.service.signal(_interface, signature="bqs")
+    def CheckerCompleted(self, success, condition, command):
+        "D-Bus signal"
+        pass
+    
+    # CheckerStarted - signal
+    @dbus.service.signal(_interface, signature="s")
+    def CheckerStarted(self, command):
+        "D-Bus signal"
+        pass
+    
+    # GetAllProperties - method
+    @dbus.service.method(_interface, out_signature="a{sv}")
+    def GetAllProperties(self):
+        "D-Bus method"
+        return dbus.Dictionary({
+                dbus.String("name"):
+                    dbus.String(self.name, variant_level=1),
+                dbus.String("fingerprint"):
+                    dbus.String(self.fingerprint, variant_level=1),
+                dbus.String("host"):
+                    dbus.String(self.host, variant_level=1),
+                dbus.String("created"):
+                    _datetime_to_dbus(self.created, variant_level=1),
+                dbus.String("last_enabled"):
+                    (_datetime_to_dbus(self.last_enabled,
+                                       variant_level=1)
+                     if self.last_enabled is not None
+                     else dbus.Boolean(False, variant_level=1)),
+                dbus.String("enabled"):
+                    dbus.Boolean(self.enabled, variant_level=1),
+                dbus.String("last_checked_ok"):
+                    (_datetime_to_dbus(self.last_checked_ok,
+                                       variant_level=1)
+                     if self.last_checked_ok is not None
+                     else dbus.Boolean (False, variant_level=1)),
+                dbus.String("timeout"):
+                    dbus.UInt64(self.timeout_milliseconds(),
+                                variant_level=1),
+                dbus.String("interval"):
+                    dbus.UInt64(self.interval_milliseconds(),
+                                variant_level=1),
+                dbus.String("checker"):
+                    dbus.String(self.checker_command,
+                                variant_level=1),
+                dbus.String("checker_running"):
+                    dbus.Boolean(self.checker is not None,
+                                 variant_level=1),
+                }, signature="sv")
+    
+    # IsStillValid - method
+    IsStillValid = (dbus.service.method(_interface, out_signature="b")
+                    (still_valid))
+    IsStillValid.__name__ = "IsStillValid"
+    
+    # PropertyChanged - signal
+    @dbus.service.signal(_interface, signature="sv")
+    def PropertyChanged(self, property, value):
+        "D-Bus signal"
+        pass
+    
+    # SetChecker - method
+    @dbus.service.method(_interface, in_signature="s")
+    def SetChecker(self, checker):
+        "D-Bus setter method"
+        self.checker_command = checker
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"checker"),
+                             dbus.String(self.checker_command,
+                                         variant_level=1))
+    
+    # SetHost - method
+    @dbus.service.method(_interface, in_signature="s")
+    def SetHost(self, host):
+        "D-Bus setter method"
+        self.host = host
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"host"),
+                             dbus.String(self.host, variant_level=1))
+    
+    # SetInterval - method
+    @dbus.service.method(_interface, in_signature="t")
+    def SetInterval(self, milliseconds):
+        self.interval = datetime.timedelta(0, 0, 0, milliseconds)
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"interval"),
+                             (dbus.UInt64(self.interval_milliseconds(),
+                                          variant_level=1)))
+    
+    # SetSecret - method
+    @dbus.service.method(_interface, in_signature="ay",
+                         byte_arrays=True)
+    def SetSecret(self, secret):
+        "D-Bus setter method"
+        self.secret = str(secret)
+    
+    # SetTimeout - method
+    @dbus.service.method(_interface, in_signature="t")
+    def SetTimeout(self, milliseconds):
+        self.timeout = datetime.timedelta(0, 0, 0, milliseconds)
+        # Emit D-Bus signal
+        self.PropertyChanged(dbus.String(u"timeout"),
+                             (dbus.UInt64(self.timeout_milliseconds(),
+                                          variant_level=1)))
+    
+    # Enable - method
+    Enable = dbus.service.method(_interface)(enable)
+    Enable.__name__ = "Enable"
+    
+    # StartChecker - method
+    @dbus.service.method(_interface)
+    def StartChecker(self):
+        "D-Bus method"
+        self.start_checker()
+    
+    # Disable - method
+    @dbus.service.method(_interface)
+    def Disable(self):
+        "D-Bus method"
+        self.disable()
+    
+    # StopChecker - method
+    StopChecker = dbus.service.method(_interface)(stop_checker)
+    StopChecker.__name__ = "StopChecker"
+    
+    del _interface
 
 
 def peer_certificate(session):
     "Return the peer's OpenPGP certificate as a bytestring"
     # If not an OpenPGP certificate...
-    if gnutls.library.functions.gnutls_certificate_type_get\
-            (session._c_object) \
-           != gnutls.library.constants.GNUTLS_CRT_OPENPGP:
+    if (gnutls.library.functions
+        .gnutls_certificate_type_get(session._c_object)
+        != gnutls.library.constants.GNUTLS_CRT_OPENPGP):
         # ...do the normal thing
         return session.peer_certificate
     list_size = ctypes.c_uint()
-    cert_list = gnutls.library.functions.gnutls_certificate_get_peers\
-        (session._c_object, ctypes.byref(list_size))
+    cert_list = (gnutls.library.functions
+                 .gnutls_certificate_get_peers
+                 (session._c_object, ctypes.byref(list_size)))
     if list_size.value == 0:
         return None
     cert = cert_list[0]
@@ -509,22 +604,24 @@
 def fingerprint(openpgp):
     "Convert an OpenPGP bytestring to a hexdigit fingerprint string"
     # New GnuTLS "datum" with the OpenPGP public key
-    datum = gnutls.library.types.gnutls_datum_t\
-        (ctypes.cast(ctypes.c_char_p(openpgp),
-                     ctypes.POINTER(ctypes.c_ubyte)),
-         ctypes.c_uint(len(openpgp)))
+    datum = (gnutls.library.types
+             .gnutls_datum_t(ctypes.cast(ctypes.c_char_p(openpgp),
+                                         ctypes.POINTER
+                                         (ctypes.c_ubyte)),
+                             ctypes.c_uint(len(openpgp))))
     # New empty GnuTLS certificate
     crt = gnutls.library.types.gnutls_openpgp_crt_t()
-    gnutls.library.functions.gnutls_openpgp_crt_init\
-        (ctypes.byref(crt))
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_init(ctypes.byref(crt)))
     # Import the OpenPGP public key into the certificate
-    gnutls.library.functions.gnutls_openpgp_crt_import\
-                    (crt, ctypes.byref(datum),
-                     gnutls.library.constants.GNUTLS_OPENPGP_FMT_RAW)
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_import(crt, ctypes.byref(datum),
+                                gnutls.library.constants
+                                .GNUTLS_OPENPGP_FMT_RAW))
     # Verify the self signature in the key
     crtverify = ctypes.c_uint()
-    gnutls.library.functions.gnutls_openpgp_crt_verify_self\
-        (crt, 0, ctypes.byref(crtverify))
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_verify_self(crt, 0, ctypes.byref(crtverify)))
     if crtverify.value != 0:
         gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
         raise gnutls.errors.CertificateSecurityError("Verify failed")
@@ -532,8 +629,9 @@
     buf = ctypes.create_string_buffer(20)
     buf_len = ctypes.c_size_t()
     # Get the fingerprint from the certificate into the buffer
-    gnutls.library.functions.gnutls_openpgp_crt_get_fingerprint\
-        (crt, ctypes.byref(buf), ctypes.byref(buf_len))
+    (gnutls.library.functions
+     .gnutls_openpgp_crt_get_fingerprint(crt, ctypes.byref(buf),
+                                         ctypes.byref(buf_len)))
     # Deinit the certificate
     gnutls.library.functions.gnutls_openpgp_crt_deinit(crt)
     # Convert the buffer to a Python bytestring
@@ -551,8 +649,10 @@
     def handle(self):
         logger.info(u"TCP connection from: %s",
                     unicode(self.client_address))
-        session = gnutls.connection.ClientSession\
-                  (self.request, gnutls.connection.X509Credentials())
+        session = (gnutls.connection
+                   .ClientSession(self.request,
+                                  gnutls.connection
+                                  .X509Credentials()))
         
         line = self.request.makefile().readline()
         logger.debug(u"Protocol version: %r", line)
@@ -573,8 +673,9 @@
         #                "+DHE-DSS"))
         # Use a fallback default, since this MUST be set.
         priority = self.server.settings.get("priority", "NORMAL")
-        gnutls.library.functions.gnutls_priority_set_direct\
-            (session._c_object, priority, None)
+        (gnutls.library.functions
+         .gnutls_priority_set_direct(session._c_object,
+                                     priority, None))
         
         try:
             session.handshake()
@@ -590,12 +691,11 @@
             session.bye()
             return
         logger.debug(u"Fingerprint: %s", fpr)
-        client = None
         for c in self.server.clients:
             if c.fingerprint == fpr:
                 client = c
                 break
-        if not client:
+        else:
             logger.warning(u"Client not found for fingerprint: %s",
                            fpr)
             session.bye()
@@ -740,14 +840,15 @@
     elif state == avahi.ENTRY_GROUP_FAILURE:
         logger.critical(u"Avahi: Error in group state changed %s",
                         unicode(error))
-        raise AvahiGroupError("State changed: %s", str(error))
+        raise AvahiGroupError(u"State changed: %s" % unicode(error))
 
 def if_nametoindex(interface):
     """Call the C function if_nametoindex(), or equivalent"""
     global if_nametoindex
     try:
-        if_nametoindex = ctypes.cdll.LoadLibrary\
-            (ctypes.util.find_library("c")).if_nametoindex
+        if_nametoindex = (ctypes.cdll.LoadLibrary
+                          (ctypes.util.find_library("c"))
+                          .if_nametoindex)
     except (OSError, AttributeError):
         if "struct" not in sys.modules:
             import struct
@@ -795,7 +896,7 @@
                       help="Address to listen for requests on")
     parser.add_option("-p", "--port", type="int",
                       help="Port number to receive requests on")
-    parser.add_option("--check", action="store_true", default=False,
+    parser.add_option("--check", action="store_true",
                       help="Run self-test")
     parser.add_option("--debug", action="store_true",
                       help="Debug mode; run in foreground and log to"
@@ -808,6 +909,10 @@
                       default="/etc/mandos", metavar="DIR",
                       help="Directory to search for configuration"
                       " files")
+    parser.add_option("--no-dbus", action="store_false",
+                      dest="use_dbus",
+                      help="Do not provide D-Bus system bus"
+                      " interface")
     options = parser.parse_args()[0]
     
     if options.check:
@@ -823,6 +928,7 @@
                         "priority":
                         "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
                         "servicename": "Mandos",
+                        "use_dbus": "True",
                         }
     
     # Parse config file for server-global settings
@@ -831,29 +937,34 @@
     server_config.read(os.path.join(options.configdir, "mandos.conf"))
     # Convert the SafeConfigParser object to a dict
     server_settings = server_config.defaults()
-    # Use getboolean on the boolean config option
-    server_settings["debug"] = server_config.getboolean\
-                               ("DEFAULT", "debug")
+    # Use getboolean on the boolean config options
+    server_settings["debug"] = (server_config.getboolean
+                                ("DEFAULT", "debug"))
+    server_settings["use_dbus"] = (server_config.getboolean
+                                   ("DEFAULT", "use_dbus"))
     del server_config
     
     # Override the settings from the config file with command line
     # options, if set.
     for option in ("interface", "address", "port", "debug",
-                   "priority", "servicename", "configdir"):
+                   "priority", "servicename", "configdir",
+                   "use_dbus"):
         value = getattr(options, option)
         if value is not None:
             server_settings[option] = value
     del options
     # Now we have our good server settings in "server_settings"
     
+    # For convenience
     debug = server_settings["debug"]
+    use_dbus = server_settings["use_dbus"]
     
     if not debug:
         syslogger.setLevel(logging.WARNING)
         console.setLevel(logging.WARNING)
     
     if server_settings["servicename"] != "Mandos":
-        syslogger.setFormatter(logging.Formatter\
+        syslogger.setFormatter(logging.Formatter
                                ('Mandos (%s): %%(levelname)s:'
                                 ' %%(message)s'
                                 % server_settings["servicename"]))
@@ -861,7 +972,7 @@
     # Parse config file with clients
     client_defaults = { "timeout": "1h",
                         "interval": "5m",
-                        "checker": "fping -q -- %(host)s",
+                        "checker": "fping -q -- %%(host)s",
                         "host": "",
                         }
     client_config = ConfigParser.SafeConfigParser(client_defaults)
@@ -880,22 +991,20 @@
     except IOError, error:
         logger.error("Could not open file %r", pidfilename)
     
-    uid = 65534
-    gid = 65534
-    try:
-        uid = pwd.getpwnam("mandos").pw_uid
-    except KeyError:
-        try:
-            uid = pwd.getpwnam("nobody").pw_uid
-        except KeyError:
-            pass
-    try:
-        gid = pwd.getpwnam("mandos").pw_gid
-    except KeyError:
-        try:
-            gid = pwd.getpwnam("nogroup").pw_gid
-        except KeyError:
-            pass
+    try:
+        uid = pwd.getpwnam("_mandos").pw_uid
+        gid = pwd.getpwnam("_mandos").pw_gid
+    except KeyError:
+        try:
+            uid = pwd.getpwnam("mandos").pw_uid
+            gid = pwd.getpwnam("mandos").pw_gid
+        except KeyError:
+            try:
+                uid = pwd.getpwnam("nobody").pw_uid
+                gid = pwd.getpwnam("nogroup").pw_gid
+            except KeyError:
+                uid = 65534
+                gid = 65534
     try:
         os.setuid(uid)
         os.setgid(gid)
@@ -907,8 +1016,8 @@
     service = AvahiService(name = server_settings["servicename"],
                            servicetype = "_mandos._tcp", )
     if server_settings["interface"]:
-        service.interface = if_nametoindex\
-                            (server_settings["interface"])
+        service.interface = (if_nametoindex
+                             (server_settings["interface"]))
     
     global main_loop
     global bus
@@ -921,21 +1030,17 @@
                                            avahi.DBUS_PATH_SERVER),
                             avahi.DBUS_INTERFACE_SERVER)
     # End of Avahi example code
-    
-    def remove_from_clients(client):
-        clients.remove(client)
-        if not clients:
-            logger.critical(u"No clients left, exiting")
-            sys.exit()
+    if use_dbus:
+        bus_name = dbus.service.BusName(u"org.mandos-system.Mandos",
+                                        bus)
     
     clients.update(Set(Client(name = section,
-                              stop_hook = remove_from_clients,
                               config
-                              = dict(client_config.items(section)))
+                              = dict(client_config.items(section)),
+                              use_dbus = use_dbus)
                        for section in client_config.sections()))
     if not clients:
-        logger.critical(u"No clients defined")
-        sys.exit(1)
+        logger.warning(u"No clients defined")
     
     if debug:
         # Redirect stdin so all checkers get /dev/null
@@ -973,8 +1078,8 @@
         
         while clients:
             client = clients.pop()
-            client.stop_hook = None
-            client.stop()
+            client.disable_hook = None
+            client.disable()
     
     atexit.register(cleanup)
     
@@ -983,8 +1088,61 @@
     signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
     signal.signal(signal.SIGTERM, lambda signum, frame: sys.exit())
     
+    if use_dbus:
+        class MandosServer(dbus.service.Object):
+            """A D-Bus proxy object"""
+            def __init__(self):
+                dbus.service.Object.__init__(self, bus,
+                                             "/Mandos")
+            _interface = u"org.mandos_system.Mandos"
+
+            @dbus.service.signal(_interface, signature="oa{sv}")
+            def ClientAdded(self, objpath, properties):
+                "D-Bus signal"
+                pass
+
+            @dbus.service.signal(_interface, signature="o")
+            def ClientRemoved(self, objpath):
+                "D-Bus signal"
+                pass
+
+            @dbus.service.method(_interface, out_signature="ao")
+            def GetAllClients(self):
+                return dbus.Array(c.dbus_object_path for c in clients)
+
+            @dbus.service.method(_interface, out_signature="a{oa{sv}}")
+            def GetAllClientsWithProperties(self):
+                return dbus.Dictionary(
+                    ((c.dbus_object_path, c.GetAllProperties())
+                     for c in clients),
+                    signature="oa{sv}")
+
+            @dbus.service.method(_interface, in_signature="o")
+            def RemoveClient(self, object_path):
+                for c in clients:
+                    if c.dbus_object_path == object_path:
+                        clients.remove(c)
+                        # Don't signal anything except ClientRemoved
+                        c.use_dbus = False
+                        c.disable()
+                        # Emit D-Bus signal
+                        self.ClientRemoved(object_path)
+                        return
+                raise KeyError
+            @dbus.service.method(_interface)
+            def Quit(self):
+                main_loop.quit()
+
+            del _interface
+    
+        mandos_server = MandosServer()
+    
     for client in clients:
-        client.start()
+        if use_dbus:
+            # Emit D-Bus signal
+            mandos_server.ClientAdded(client.dbus_object_path,
+                                      client.GetAllProperties())
+        client.enable()
     
     tcp_server.enable()
     tcp_server.server_activate()
@@ -1008,13 +1166,13 @@
         
         gobject.io_add_watch(tcp_server.fileno(), gobject.IO_IN,
                              lambda *args, **kwargs:
-                             tcp_server.handle_request\
-                             (*args[2:], **kwargs) or True)
+                             (tcp_server.handle_request
+                              (*args[2:], **kwargs) or True))
         
         logger.debug(u"Starting main loop")
         main_loop.run()
     except AvahiError, error:
-        logger.critical(u"AvahiError: %s" + unicode(error))
+        logger.critical(u"AvahiError: %s", error)
         sys.exit(1)
     except KeyboardInterrupt:
         if debug:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2008-10-07 21:31:09 +0000
+++ mandos-clients.conf.xml	2009-01-08 03:54:06 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2008-10-07">
+<!ENTITY TIMESTAMP "2009-01-08">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -169,7 +170,7 @@
 	    <varname>PATH</varname> will be searched.  The default
 	    value for the checker command is <quote><literal
 	    ><command>fping</command> <option>-q</option> <option
-	    >--</option> %(host)s</literal></quote>.
+	    >--</option> %%(host)s</literal></quote>.
 	  </para>
 	  <para>
 	    In addition to normal start time expansion, this option
@@ -350,7 +351,7 @@
 [DEFAULT]
 timeout = 1h
 interval = 5m
-checker = fping -q -- %(host)s
+checker = fping -q -- %%(host)s
 
 # Client "foo"
 [foo]

=== modified file 'mandos-keygen'
--- mandos-keygen	2008-10-17 18:56:25 +0000
+++ mandos-keygen	2009-01-06 02:42:53 +0000
@@ -2,7 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2008 Teddy Hogeborn & Björn Påhlsson
+# Copyright © 2008,2009 Teddy Hogeborn
+# Copyright © 2008,2009 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,7 +21,7 @@
 # Contact the authors at <mandos@fukt.bsnet.se>.
 # 
 
-VERSION="1.0.2"
+VERSION="1.0.3"
 
 KEYDIR="/etc/keys/mandos"
 KEYTYPE=DSA

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2008-10-04 01:55:56 +0000
+++ mandos-keygen.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2008-10-03">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== added file 'mandos-list'
--- mandos-list	1970-01-01 00:00:00 +0000
+++ mandos-list	2008-12-21 19:19:25 +0000
@@ -0,0 +1,65 @@
+#!/usr/bin/python
+# -*- mode: python; coding: utf-8 -*-
+
+import dbus
+from optparse import OptionParser
+import locale
+
+locale.setlocale(locale.LC_ALL, u'')
+
+tablewords = {
+    'name': u'Name',
+    'enabled': u'Enabled',
+    'timeout': u'Timeout',
+    'last_checked_ok': u'Last Successful Check',
+    'created': u'Created',
+    'interval': u'Interval',
+    'host': u'Host',
+    'fingerprint': u'Fingerprint',
+    'checker_running': u'Check Is Running',
+    'last_enabled': u'Last Enabled',
+    'checker': u'Checker',
+    }
+busname = 'org.mandos-system.Mandos'
+object_path = '/Mandos'
+interface = 'org.mandos_system.Mandos'
+version = "1.0.2"
+defaultkeywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                   'checker')
+
+parser = OptionParser(version = "%%prog %s" % version)
+parser.add_option("-a", "--all", action="store_true", default=False,
+                  help="Print all fields")
+options = parser.parse_args()[0]
+if options.all:
+    keywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                'created', 'interval', 'host', 'fingerprint',
+                'checker_running', 'last_enabled', 'checker')
+else:
+    keywords = defaultkeywords
+
+
+bus = dbus.SystemBus()
+mandos_dbus_objc = bus.get_object(busname, object_path)
+mandos_serv = dbus.Interface(mandos_dbus_objc,
+                             dbus_interface = interface)
+mandos_clients = mandos_serv.GetAllClientsWithProperties()
+
+def valuetostring(x):
+    if type(x) is dbus.Boolean:
+        return u"Yes" if x else u"No"
+    else:
+        return unicode(x)
+
+format_string = u' '.join(u'%%-%ds'
+                         % max(len(tablewords[key]),
+                               max(len(valuetostring(client[key]))
+                                   for client
+                                   in mandos_clients.itervalues()))
+                         for key in keywords)
+print format_string % tuple(tablewords[key] for key in keywords) 
+for client in mandos_clients.itervalues():
+    print format_string % tuple(valuetostring(client[key])
+                                for key in keywords)
+
+

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2008-09-30 03:19:39 +0000
+++ mandos-options.xml	2008-12-29 02:44:54 +0000
@@ -65,5 +65,11 @@
     rename itself to <quote><literal>Mandos #2</literal></quote>, and
     so on; therefore, this option is not needed in that case.
   </para>
+
+  <para id="dbus">
+    This option controls whether the server will provide a D-Bus
+    system bus interface.  The default is to provide such an
+    interface.
+  </para>
   
 </section>

=== modified file 'mandos.conf'
--- mandos.conf	2008-08-18 23:55:28 +0000
+++ mandos.conf	2008-12-29 02:44:54 +0000
@@ -36,3 +36,6 @@
 # If there are name collisions on the same *network*, the server will
 # rename itself to "Mandos #2", etc.
 ;servicename = Mandos
+
+# Whether to provide a D-Bus system bus interface or not
+;use_dbus = True

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2008-09-30 07:23:39 +0000
+++ mandos.conf.xml	2009-01-04 21:54:55 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2008-09-30">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -33,6 +33,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -130,6 +131,17 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>use_dbus<literal> = </literal>{ <literal
+	  >1</literal> | <literal>yes</literal> | <literal
+	  >true</literal> | <literal>on</literal> | <literal
+	  >0</literal> | <literal>no</literal> | <literal
+	  >false</literal> | <literal>off</literal> }</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="dbus"/>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   
@@ -172,6 +184,7 @@
 debug = true
 priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
 servicename = Daena
+use_dbus = False
       </programlisting>
     </informalexample>
   </refsect1>

=== modified file 'mandos.lsm'
--- mandos.lsm	2008-10-18 10:58:29 +0000
+++ mandos.lsm	2009-01-06 02:42:53 +0000
@@ -1,7 +1,7 @@
 Begin4
 Title:		Mandos
-Version:	1.0.2
-Entered-date:	2008-10-18
+Version:	1.0.3
+Entered-date:	2009-01-06
 Description:	The Mandos system allows computers to have encrypted
 root file systems and at the same time be capable of remote and/or
 unattended reboots.
@@ -12,9 +12,9 @@
 Maintained-by:	teddy@fukt.bsnet.se (Teddy Hogeborn),
 		belorn@fukt.bsnet.se (Björn Påhlsson)
 Primary-site: http://www.fukt.bsnet.se/mandos
-	      89K mandos_1.0.2.orig.tar.gz
+	      92K mandos_1.0.3.orig.tar.gz
 Alternate-site: ftp://ftp.fukt.bsnet.se/pub/mandos
-		89K mandos_1.0.2.orig.tar.gz
+		92K mandos_1.0.3.orig.tar.gz
 Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.4, and
 various other libraries.  While made for Debian GNU/Linux, it is
 probably portable to other distributions, but not other Unixes.

=== modified file 'mandos.xml'
--- mandos.xml	2008-10-04 01:55:56 +0000
+++ mandos.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2008-10-03">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -84,6 +85,8 @@
       <replaceable>DIRECTORY</replaceable></option></arg>
       <sbr/>
       <arg><option>--debug</option></arg>
+      <sbr/>
+      <arg><option>--no-dbus</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -228,6 +231,16 @@
 	  </para>
 	</listitem>
       </varlistentry>
+      
+      <varlistentry>
+	<term><option>--no-dbus</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml" xpointer="dbus"/>
+	  <para>
+	    See also <xref linkend="dbus_interface"/>.
+	  </para>
+	</listitem>
+      </varlistentry>
     </variablelist>
   </refsect1>
   
@@ -323,6 +336,16 @@
     </para>
   </refsect1>
   
+  <refsect1 id="dbus_interface">
+    <title>D-BUS INTERFACE</title>
+    <para>
+      The server will by default provide a D-Bus system bus interface.
+      This interface will only be accessible by the root user or a
+      Mandos-specific user, if such a user exists.
+      <!-- XXX -->
+    </para>
+  </refsect1>
+
   <refsect1 id="exit_status">
     <title>EXIT STATUS</title>
     <para>

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2008-11-01 02:26:00 +0000
+++ plugin-runner.c	2009-01-14 14:20:17 +0000
@@ -2,7 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008 Teddy Hogeborn & Björn Påhlsson
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -63,6 +64,7 @@
 				   sigprocmask(), SIG_BLOCK, SIGCHLD,
 				   SIG_UNBLOCK, kill() */
 #include <errno.h>		/* errno, EBADF */
+#include <inttypes.h>		/* intmax_t, SCNdMAX, PRIdMAX,  */
 
 #define BUFFER_SIZE 256
 
@@ -98,15 +100,15 @@
    or if none is found, creates a new one */
 static plugin *getplugin(char *name){
   /* Check for exiting plugin with that name */
-  for (plugin *p = plugin_list; p != NULL; p = p->next){
-    if ((p->name == name)
-	or (p->name and name and (strcmp(p->name, name) == 0))){
+  for(plugin *p = plugin_list; p != NULL; p = p->next){
+    if((p->name == name)
+       or (p->name and name and (strcmp(p->name, name) == 0))){
       return p;
     }
   }
   /* Create a new plugin */
   plugin *new_plugin = malloc(sizeof(plugin));
-  if (new_plugin == NULL){
+  if(new_plugin == NULL){
     return NULL;
   }
   char *copy_name = NULL;
@@ -123,7 +125,7 @@
 			   .next = plugin_list };
   
   new_plugin->argv = malloc(sizeof(char *) * 2);
-  if (new_plugin->argv == NULL){
+  if(new_plugin->argv == NULL){
     free(copy_name);
     free(new_plugin);
     return NULL;
@@ -229,7 +231,7 @@
       break;
     }
     if(pid == -1){
-      if (errno != ECHILD){
+      if(errno != ECHILD){
 	perror("waitpid");
       }
       /* No child processes */
@@ -307,7 +309,9 @@
   struct dirent *dirst;
   struct stat st;
   fd_set rfds_all;
-  int ret, maxfd = 0;
+  int ret, numchars, maxfd = 0;
+  ssize_t sret;
+  intmax_t tmpmax;
   uid_t uid = 65534;
   gid_t gid = 65534;
   bool debug = false;
@@ -370,11 +374,11 @@
     { .name = NULL }
   };
   
-  error_t parse_opt (int key, char *arg, __attribute__((unused))
-		     struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt(int key, char *arg, __attribute__((unused))
+		    struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p;
 	while((p = strsep(&arg, ",")) != NULL){
 	  if(p[0] == '\0'){
@@ -396,7 +400,7 @@
       }
       break;
     case 'o':			/* --options-for */
-      if (arg != NULL){
+      if(arg != NULL){
 	char *p_name = strsep(&arg, ":");
 	if(p_name[0] == '\0' or arg == NULL){
 	  break;
@@ -433,7 +437,7 @@
       }
       break;
     case 'd':			/* --disable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -442,7 +446,7 @@
       }
       break;
     case 'e':			/* --enable */
-      if (arg != NULL){
+      if(arg != NULL){
 	plugin *p = getplugin(arg);
 	if(p == NULL){
 	  return ARGP_ERR_UNKNOWN;
@@ -461,14 +465,32 @@
       /* This is already done by parse_opt_config_file() */
       break;
     case 130:			/* --userid */
-      uid = (uid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad user ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)uid);
+      } else {
+	uid = (uid_t)tmpmax;
+      }
       break;
     case 131:			/* --groupid */
-      gid = (gid_t)strtol(arg, NULL, 10);
+      ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (gid_t)tmpmax
+	 or arg[numchars] != '\0'){
+	fprintf(stderr, "Bad group ID number: \"%s\", using %"
+		PRIdMAX "\n", arg, (intmax_t)gid);
+      } else {
+	gid = (gid_t)tmpmax;
+      }
       break;
     case 132:			/* --debug */
       debug = true;
       break;
+/*
+ * When adding more options before this line, remember to also add a
+ * "case" to the "parse_opt_config_file" function below.
+ */
     case ARGP_KEY_ARG:
       /* Cryptsetup always passes an argument, which is an empty
 	 string if "none" was specified in /etc/crypttab.  So if
@@ -487,10 +509,10 @@
   
   /* This option parser is the same as parse_opt() above, except it
      ignores everything but the --config-file option. */
-  error_t parse_opt_config_file (int key, char *arg,
-				 __attribute__((unused))
-				 struct argp_state *state) {
-    switch (key) {
+  error_t parse_opt_config_file(int key, char *arg,
+				__attribute__((unused))
+				struct argp_state *state) {
+    switch(key) {
     case 'g': 			/* --global-options */
     case 'G':			/* --global-env */
     case 'o':			/* --options-for */
@@ -523,10 +545,10 @@
 		       .args_doc = "",
 		       .doc = "Mandos plugin runner -- Run plugins" };
   
-  /* Parse using the parse_opt_config_file in order to get the custom
+  /* Parse using parse_opt_config_file() in order to get the custom
      config file location, if any. */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -536,7 +558,7 @@
   argp.parser = parse_opt;
   
   /* Open the configfile if available */
-  if (argfile == NULL){
+  if(argfile == NULL){
     conffp = fopen(AFILE, "r");
   } else {
     conffp = fopen(argfile, "r");
@@ -545,7 +567,6 @@
     char *org_line = NULL;
     char *p, *arg, *new_arg, *line;
     size_t size = 0;
-    ssize_t sret;
     const char whitespace_delims[] = " \r\t\f\v\n";
     const char comment_delim[] = "#";
 
@@ -598,7 +619,7 @@
   } else {
     /* Check for harmful errors and go to fallback. Other errors might
        not affect opening plugins */
-    if (errno == EMFILE or errno == ENFILE or errno == ENOMEM){
+    if(errno == EMFILE or errno == ENFILE or errno == ENOMEM){
       perror("fopen");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -607,9 +628,9 @@
   /* If there was any arguments from configuration file,
      pass them to parser as command arguments */
   if(custom_argv != NULL){
-    ret = argp_parse (&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
-		      0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, custom_argc, custom_argv, ARGP_IN_ORDER,
+		     0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -618,8 +639,8 @@
   
   /* Parse actual command line arguments, to let them override the
      config file */
-  ret = argp_parse (&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
-  if (ret == ARGP_ERR_UNKNOWN){
+  ret = argp_parse(&argp, argc, argv, ARGP_IN_ORDER, 0, NULL);
+  if(ret == ARGP_ERR_UNKNOWN){
     fprintf(stderr, "Unknown error while parsing arguments\n");
     exitstatus = EXIT_FAILURE;
     goto fallback;
@@ -632,7 +653,7 @@
       for(char **a = p->argv; *a != NULL; a++){
 	fprintf(stderr, "\tArg: %s\n", *a);
       }
-      fprintf(stderr, "...and %u environment variables\n", p->envc);
+      fprintf(stderr, "...and %d environment variables\n", p->envc);
       for(char **a = p->environ; *a != NULL; a++){
 	fprintf(stderr, "\t%s\n", *a);
       }
@@ -641,15 +662,15 @@
   
   /* Strip permissions down to nobody */
   ret = setuid(uid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setuid");
   }  
   setgid(gid);
-  if (ret == -1){
+  if(ret == -1){
     perror("setgid");
   }
   
-  if (plugindir == NULL){
+  if(plugindir == NULL){
     dir = opendir(PDIR);
   } else {
     dir = opendir(plugindir);
@@ -682,7 +703,7 @@
     
     /* All directory entries have been processed */
     if(dirst == NULL){
-      if (errno == EBADF){
+      if(errno == EBADF){
 	perror("readdir");
 	exitstatus = EXIT_FAILURE;
 	goto fallback;
@@ -748,14 +769,14 @@
     }
     
     ret = stat(filename, &st);
-    if (ret == -1){
+    if(ret == -1){
       perror("stat");
       free(filename);
       continue;
     }
 
     /* Ignore non-executable files */
-    if (not S_ISREG(st.st_mode)	or (access(filename, X_OK) != 0)){
+    if(not S_ISREG(st.st_mode) or (access(filename, X_OK) != 0)){
       if(debug){
 	fprintf(stderr, "Ignoring plugin dir entry \"%s\""
 		" with bad type or mode\n", filename);
@@ -808,7 +829,7 @@
     
     int pipefd[2];
     ret = pipe(pipefd);
-    if (ret == -1){
+    if(ret == -1){
       perror("pipe");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -827,7 +848,7 @@
       goto fallback;
     }
     /* Block SIGCHLD until process is safely in process list */
-    ret = sigprocmask (SIG_BLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_BLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -881,9 +902,9 @@
     close(pipefd[1]);		/* Close unused write end of pipe */
     free(filename);
     plugin *new_plugin = getplugin(dirst->d_name);
-    if (new_plugin == NULL){
+    if(new_plugin == NULL){
       perror("getplugin");
-      ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+      ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
       if(ret < 0){
         perror("sigprocmask");
       }
@@ -896,7 +917,7 @@
     
     /* Unblock SIGCHLD so signal handler can be run if this process
        has already completed */
-    ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
+    ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask, NULL);
     if(ret < 0){
       perror("sigprocmask");
       exitstatus = EXIT_FAILURE;
@@ -905,7 +926,7 @@
     
     FD_SET(new_plugin->fd, &rfds_all);
     
-    if (maxfd < new_plugin->fd){
+    if(maxfd < new_plugin->fd){
       maxfd = new_plugin->fd;
     }
   }
@@ -928,7 +949,7 @@
   while(plugin_list){
     fd_set rfds = rfds_all;
     int select_ret = select(maxfd+1, &rfds, NULL, NULL, NULL);
-    if (select_ret == -1){
+    if(select_ret == -1){
       perror("select");
       exitstatus = EXIT_FAILURE;
       goto fallback;
@@ -945,16 +966,16 @@
 
 	  if(debug){
 	    if(WIFEXITED(proc->status)){
-	      fprintf(stderr, "Plugin %u exited with status %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " exited with status"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WEXITSTATUS(proc->status));
 	    } else if(WIFSIGNALED(proc->status)) {
-	      fprintf(stderr, "Plugin %u killed by signal %d\n",
-		      (unsigned int) (proc->pid),
+	      fprintf(stderr, "Plugin %" PRIdMAX " killed by signal"
+		      " %d\n", (intmax_t) (proc->pid),
 		      WTERMSIG(proc->status));
 	    } else if(WCOREDUMP(proc->status)){
-	      fprintf(stderr, "Plugin %d dumped core\n",
-		      (unsigned int) (proc->pid));
+	      fprintf(stderr, "Plugin %" PRIdMAX " dumped core\n",
+		      (intmax_t) (proc->pid));
 	    }
 	  }
 	  
@@ -974,8 +995,8 @@
 	  proc = next_plugin;
 	  
 	  /* We are done modifying process list, so unblock signal */
-	  ret = sigprocmask (SIG_UNBLOCK, &sigchld_action.sa_mask,
-			     NULL);
+	  ret = sigprocmask(SIG_UNBLOCK, &sigchld_action.sa_mask,
+			    NULL);
 	  if(ret < 0){
 	    perror("sigprocmask");
 	    exitstatus = EXIT_FAILURE;
@@ -1010,7 +1031,7 @@
       if(proc->buffer_length + BUFFER_SIZE > proc->buffer_size){
 	proc->buffer = realloc(proc->buffer, proc->buffer_size
 			       + (size_t) BUFFER_SIZE);
-	if (proc->buffer == NULL){
+	if(proc->buffer == NULL){
 	  perror("malloc");
 	  exitstatus = EXIT_FAILURE;
 	  goto fallback;
@@ -1018,18 +1039,18 @@
 	proc->buffer_size += BUFFER_SIZE;
       }
       /* Read from the process */
-      ret = read(proc->fd, proc->buffer + proc->buffer_length,
-		 BUFFER_SIZE);
-      if(ret < 0){
+      sret = read(proc->fd, proc->buffer + proc->buffer_length,
+		  BUFFER_SIZE);
+      if(sret < 0){
 	/* Read error from this process; ignore the error */
 	proc = proc->next;
 	continue;
       }
-      if(ret == 0){
+      if(sret == 0){
 	/* got EOF */
 	proc->eof = true;
       } else {
-	proc->buffer_length += (size_t) ret;
+	proc->buffer_length += (size_t) sret;
       }
     }
   }

=== modified file 'plugin-runner.xml'
--- plugin-runner.xml	2008-09-30 07:23:39 +0000
+++ plugin-runner.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "plugin-runner">
-<!ENTITY TIMESTAMP "2008-09-30">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/askpass-fifo.c'
--- plugins.d/askpass-fifo.c	2008-09-26 19:47:21 +0000
+++ plugins.d/askpass-fifo.c	2009-01-10 06:00:50 +0000
@@ -1,3 +1,28 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Askpass-FIFO - Read a password from a FIFO and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
 #include <sys/types.h>		/* ssize_t */
 #include <sys/stat.h>		/* mkfifo(), S_IRUSR, S_IWUSR */
@@ -18,14 +43,14 @@
   
   /* Create FIFO */
   const char passfifo[] = "/lib/cryptsetup/passfifo";
-  ret = TEMP_FAILURE_RETRY(mkfifo(passfifo, S_IRUSR | S_IWUSR));
+  ret = (int)TEMP_FAILURE_RETRY(mkfifo(passfifo, S_IRUSR | S_IWUSR));
   if(ret == -1 and errno != EEXIST){
     perror("mkfifo");
     return EXIT_FAILURE;
   }
   
   /* Open FIFO */
-  int fifo_fd = TEMP_FAILURE_RETRY(open(passfifo, O_RDONLY));
+  int fifo_fd = (int)TEMP_FAILURE_RETRY(open(passfifo, O_RDONLY));
   if(fifo_fd == -1){
     perror("open");
     return EXIT_FAILURE;

=== modified file 'plugins.d/askpass-fifo.xml'
--- plugins.d/askpass-fifo.xml	2008-10-04 20:09:53 +0000
+++ plugins.d/askpass-fifo.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "askpass-fifo">
-<!ENTITY TIMESTAMP "2008-10-04">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2008-09-30 07:23:39 +0000
+++ plugins.d/mandos-client.c	2009-01-14 14:20:17 +0000
@@ -1,6 +1,6 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Mandos client - get and decrypt data from a Mandos server
+ * Mandos-client - get and decrypt data from a Mandos server
  *
  * This program is partly derived from an example program for an Avahi
  * service browser, downloaded from
@@ -9,7 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008 Teddy Hogeborn & Björn Påhlsson
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,7 +36,7 @@
 #define _GNU_SOURCE		/* TEMP_FAILURE_RETRY(), asprintf() */
 
 #include <stdio.h>		/* fprintf(), stderr, fwrite(),
-				   stdout, ferror() */
+				   stdout, ferror(), sscanf */
 #include <stdint.h> 		/* uint16_t, uint32_t */
 #include <stddef.h>		/* NULL, size_t, ssize_t */
 #include <stdlib.h> 		/* free(), EXIT_SUCCESS, EXIT_FAILURE,
@@ -47,14 +48,16 @@
 #include <sys/types.h>		/* socket(), inet_pton(), sockaddr,
 				   sockaddr_in6, PF_INET6,
 				   SOCK_STREAM, INET6_ADDRSTRLEN,
-				   uid_t, gid_t, open(), opendir(), DIR */
+				   uid_t, gid_t, open(), opendir(),
+				   DIR */
 #include <sys/stat.h>		/* open() */
 #include <sys/socket.h>		/* socket(), struct sockaddr_in6,
 				   struct in6_addr, inet_pton(),
 				   connect() */
 #include <fcntl.h>		/* open() */
-#include <dirent.h>		/* opendir(), struct dirent, readdir() */
-#include <inttypes.h>		/* PRIu16 */
+#include <dirent.h>		/* opendir(), struct dirent, readdir()
+				 */
+#include <inttypes.h>		/* PRIu16, intmax_t, SCNdMAX */
 #include <assert.h>		/* assert() */
 #include <errno.h>		/* perror(), errno */
 #include <time.h>		/* time() */
@@ -66,7 +69,7 @@
 				   getuid(), getgid(), setuid(),
 				   setgid() */
 #include <arpa/inet.h>		/* inet_pton(), htons */
-#include <iso646.h>		/* not, and */
+#include <iso646.h>		/* not, and, or */
 #include <argp.h>		/* struct argp_option, error_t, struct
 				   argp_state, struct argp,
 				   argp_parse(), ARGP_KEY_ARG,
@@ -89,7 +92,8 @@
 				   gnutls_*
 				   init_gnutls_session(),
 				   GNUTLS_* */
-#include <gnutls/openpgp.h>     /* gnutls_certificate_set_openpgp_key_file(),
+#include <gnutls/openpgp.h>
+			  /* gnutls_certificate_set_openpgp_key_file(),
 				   GNUTLS_OPENPGP_FMT_BASE64 */
 
 /* GPGME */
@@ -128,9 +132,9 @@
  */
 size_t adjustbuffer(char **buffer, size_t buffer_length,
 		  size_t buffer_capacity){
-  if (buffer_length + BUFFER_SIZE > buffer_capacity){
+  if(buffer_length + BUFFER_SIZE > buffer_capacity){
     *buffer = realloc(*buffer, buffer_capacity + BUFFER_SIZE);
-    if (buffer == NULL){
+    if(buffer == NULL){
       return 0;
     }
     buffer_capacity += BUFFER_SIZE;
@@ -155,27 +159,27 @@
     int fd;
     gpgme_data_t pgp_data;
     
-    fd = TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
+    fd = (int)TEMP_FAILURE_RETRY(open(filename, O_RDONLY));
     if(fd == -1){
       perror("open");
       return false;
     }
     
     rc = gpgme_data_new_from_fd(&pgp_data, fd);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_data_new_from_fd: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
     rc = gpgme_op_import(mc->ctx, pgp_data);
-    if (rc != GPG_ERR_NO_ERROR){
+    if(rc != GPG_ERR_NO_ERROR){
       fprintf(stderr, "bad gpgme_op_import: %s: %s\n",
 	      gpgme_strsource(rc), gpgme_strerror(rc));
       return false;
     }
     
-    ret = TEMP_FAILURE_RETRY(close(fd));
+    ret = (int)TEMP_FAILURE_RETRY(close(fd));
     if(ret == -1){
       perror("close");
     }
@@ -183,22 +187,22 @@
     return true;
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Initialize gpgme\n");
   }
   
   /* Init GPGME */
   gpgme_check_version(NULL);
   rc = gpgme_engine_check_version(GPGME_PROTOCOL_OpenPGP);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_engine_check_version: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
     /* Set GPGME home directory for the OpenPGP engine only */
-  rc = gpgme_get_engine_info (&engine_info);
-  if (rc != GPG_ERR_NO_ERROR){
+  rc = gpgme_get_engine_info(&engine_info);
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_get_engine_info: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
@@ -218,13 +222,13 @@
   
   /* Create new GPGME "context" */
   rc = gpgme_new(&(mc->ctx));
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return false;
   }
   
-  if (not import_key(pubkey) or not import_key(seckey)){
+  if(not import_key(pubkey) or not import_key(seckey)){
     return false;
   }
   
@@ -235,24 +239,24 @@
  * Decrypt OpenPGP data.
  * Returns -1 on error
  */
-static ssize_t pgp_packet_decrypt (const mandos_context *mc,
-				   const char *cryptotext,
-				   size_t crypto_size,
-				   char **plaintext){
+static ssize_t pgp_packet_decrypt(const mandos_context *mc,
+				  const char *cryptotext,
+				  size_t crypto_size,
+				  char **plaintext){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
   size_t plaintext_capacity = 0;
   ssize_t plaintext_length = 0;
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Trying to decrypt OpenPGP data\n");
   }
   
   /* Create new GPGME data buffer from memory cryptotext */
   rc = gpgme_data_new_from_mem(&dh_crypto, cryptotext, crypto_size,
 			       0);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new_from_mem: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     return -1;
@@ -260,7 +264,7 @@
   
   /* Create new empty GPGME data buffer for the plaintext */
   rc = gpgme_data_new(&dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_data_new: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     gpgme_data_release(dh_crypto);
@@ -270,14 +274,14 @@
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
   rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
-  if (rc != GPG_ERR_NO_ERROR){
+  if(rc != GPG_ERR_NO_ERROR){
     fprintf(stderr, "bad gpgme_op_decrypt: %s: %s\n",
 	    gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
-    if (debug){
+    if(debug){
       gpgme_decrypt_result_t result;
       result = gpgme_op_decrypt_result(mc->ctx);
-      if (result == NULL){
+      if(result == NULL){
 	fprintf(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
 	fprintf(stderr, "Unsupported algorithm: %s\n",
@@ -310,7 +314,7 @@
   }
   
   /* Seek back to the beginning of the GPGME plaintext data buffer */
-  if (gpgme_data_seek(dh_plain, (off_t) 0, SEEK_SET) == -1){
+  if(gpgme_data_seek(dh_plain, (off_t)0, SEEK_SET) == -1){
     perror("gpgme_data_seek");
     plaintext_length = -1;
     goto decrypt_end;
@@ -321,7 +325,7 @@
     plaintext_capacity = adjustbuffer(plaintext,
 				      (size_t)plaintext_length,
 				      plaintext_capacity);
-    if (plaintext_capacity == 0){
+    if(plaintext_capacity == 0){
 	perror("adjustbuffer");
 	plaintext_length = -1;
 	goto decrypt_end;
@@ -330,7 +334,7 @@
     ret = gpgme_data_read(dh_plain, *plaintext + plaintext_length,
 			  BUFFER_SIZE);
     /* Print the data, if any */
-    if (ret == 0){
+    if(ret == 0){
       /* EOF */
       break;
     }
@@ -360,9 +364,10 @@
   return plaintext_length;
 }
 
-static const char * safer_gnutls_strerror (int value) {
-  const char *ret = gnutls_strerror (value); /* Spurious warning */
-  if (ret == NULL)
+static const char * safer_gnutls_strerror(int value) {
+  const char *ret = gnutls_strerror(value); /* Spurious warning from
+					       -Wunreachable-code */
+  if(ret == NULL)
     ret = "(unknown)";
   return ret;
 }
@@ -383,13 +388,13 @@
   }
   
   ret = gnutls_global_init();
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "GnuTLS global_init: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "GnuTLS global_init: %s\n",
+	    safer_gnutls_strerror(ret));
     return -1;
   }
   
-  if (debug){
+  if(debug){
     /* "Use a log level over 10 to enable all debugging options."
      * - GnuTLS manual
      */
@@ -399,11 +404,13 @@
   
   /* OpenPGP credentials */
   gnutls_certificate_allocate_credentials(&mc->cred);
-  if (ret != GNUTLS_E_SUCCESS){
-    fprintf (stderr, "GnuTLS memory error: %s\n", /* Spurious
-						     warning */
-	     safer_gnutls_strerror(ret));
-    gnutls_global_deinit ();
+  if(ret != GNUTLS_E_SUCCESS){
+    fprintf(stderr, "GnuTLS memory error: %s\n", /* Spurious warning
+						  * from
+						  * -Wunreachable-code
+						  */
+	    safer_gnutls_strerror(ret));
+    gnutls_global_deinit();
     return -1;
   }
   
@@ -416,7 +423,7 @@
   ret = gnutls_certificate_set_openpgp_key_file
     (mc->cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr,
 	    "Error[%d] while reading the OpenPGP key pair ('%s',"
 	    " '%s')\n", ret, pubkeyfilename, seckeyfilename);
@@ -427,15 +434,15 @@
   
   /* GnuTLS server initialization */
   ret = gnutls_dh_params_init(&mc->dh_params);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS DH parameter initialization:"
-	     " %s\n", safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS DH parameter initialization:"
+	    " %s\n", safer_gnutls_strerror(ret));
     goto globalfail;
   }
   ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
-  if (ret != GNUTLS_E_SUCCESS) {
-    fprintf (stderr, "Error in GnuTLS prime generation: %s\n",
-	     safer_gnutls_strerror(ret));
+  if(ret != GNUTLS_E_SUCCESS) {
+    fprintf(stderr, "Error in GnuTLS prime generation: %s\n",
+	    safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
@@ -456,7 +463,7 @@
   int ret;
   /* GnuTLS session creation */
   ret = gnutls_init(session, GNUTLS_SERVER);
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     fprintf(stderr, "Error in GnuTLS session initialization: %s\n",
 	    safer_gnutls_strerror(ret));
   }
@@ -464,29 +471,29 @@
   {
     const char *err;
     ret = gnutls_priority_set_direct(*session, mc->priority, &err);
-    if (ret != GNUTLS_E_SUCCESS) {
+    if(ret != GNUTLS_E_SUCCESS) {
       fprintf(stderr, "Syntax error at: %s\n", err);
       fprintf(stderr, "GnuTLS error: %s\n",
 	      safer_gnutls_strerror(ret));
-      gnutls_deinit (*session);
+      gnutls_deinit(*session);
       return -1;
     }
   }
   
   ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
 			       mc->cred);
-  if (ret != GNUTLS_E_SUCCESS) {
+  if(ret != GNUTLS_E_SUCCESS) {
     fprintf(stderr, "Error setting GnuTLS credentials: %s\n",
 	    safer_gnutls_strerror(ret));
-    gnutls_deinit (*session);
+    gnutls_deinit(*session);
     return -1;
   }
   
   /* ignore client certificate if any. */
-  gnutls_certificate_server_set_request (*session,
-					 GNUTLS_CERT_IGNORE);
+  gnutls_certificate_server_set_request(*session,
+					GNUTLS_CERT_IGNORE);
   
-  gnutls_dh_set_prime_bits (*session, mc->dh_bits);
+  gnutls_dh_set_prime_bits(*session, mc->dh_bits);
   
   return 0;
 }
@@ -500,6 +507,7 @@
 				      AvahiIfIndex if_index,
 				      mandos_context *mc){
   int ret, tcp_sd;
+  ssize_t sret;
   union { struct sockaddr in; struct sockaddr_in6 in6; } to;
   char *buffer = NULL;
   char *decrypted_buffer;
@@ -511,8 +519,8 @@
   char interface[IF_NAMESIZE];
   gnutls_session_t session;
   
-  ret = init_gnutls_session (mc, &session);
-  if (ret != 0){
+  ret = init_gnutls_session(mc, &session);
+  if(ret != 0){
     return -1;
   }
   
@@ -540,7 +548,7 @@
   /* It would be nice to have a way to detect if we were passed an
      IPv4 address here.   Now we assume an IPv6 address. */
   ret = inet_pton(AF_INET6, ip, &to.in6.sin6_addr);
-  if (ret < 0 ){
+  if(ret < 0 ){
     perror("inet_pton");
     return -1;
   }
@@ -548,7 +556,9 @@
     fprintf(stderr, "Bad address: %s\n", ip);
     return -1;
   }
-  to.in6.sin6_port = htons(port); /* Spurious warning */
+  to.in6.sin6_port = htons(port); /* Spurious warnings from
+				     -Wconversion and
+				     -Wunreachable-code */
   
   to.in6.sin6_scope_id = (uint32_t)if_index;
   
@@ -567,18 +577,18 @@
   }
   
   ret = connect(tcp_sd, &to.in, sizeof(to));
-  if (ret < 0){
+  if(ret < 0){
     perror("connect");
     return -1;
   }
   
   const char *out = mandos_protocol_version;
   written = 0;
-  while (true){
+  while(true){
     size_t out_size = strlen(out);
-    ret = TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
+    ret = (int)TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
 				   out_size - written));
-    if (ret == -1){
+    if(ret == -1){
       perror("write");
       retval = -1;
       goto mandos_end;
@@ -587,7 +597,7 @@
     if(written < out_size){
       continue;
     } else {
-      if (out == mandos_protocol_version){
+      if(out == mandos_protocol_version){
 	written = 0;
 	out = "\r\n";
       } else {
@@ -600,16 +610,16 @@
     fprintf(stderr, "Establishing TLS session with %s\n", ip);
   }
   
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) tcp_sd);
+  gnutls_transport_set_ptr(session, (gnutls_transport_ptr_t) tcp_sd);
   
   do{
-    ret = gnutls_handshake (session);
+    ret = gnutls_handshake(session);
   } while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
   
-  if (ret != GNUTLS_E_SUCCESS){
+  if(ret != GNUTLS_E_SUCCESS){
     if(debug){
       fprintf(stderr, "*** GnuTLS Handshake failed ***\n");
-      gnutls_perror (ret);
+      gnutls_perror(ret);
     }
     retval = -1;
     goto mandos_end;
@@ -625,29 +635,29 @@
   while(true){
     buffer_capacity = adjustbuffer(&buffer, buffer_length,
 				   buffer_capacity);
-    if (buffer_capacity == 0){
+    if(buffer_capacity == 0){
       perror("adjustbuffer");
       retval = -1;
       goto mandos_end;
     }
     
-    ret = gnutls_record_recv(session, buffer+buffer_length,
-			     BUFFER_SIZE);
-    if (ret == 0){
+    sret = gnutls_record_recv(session, buffer+buffer_length,
+			      BUFFER_SIZE);
+    if(sret == 0){
       break;
     }
-    if (ret < 0){
-      switch(ret){
+    if(sret < 0){
+      switch(sret){
       case GNUTLS_E_INTERRUPTED:
       case GNUTLS_E_AGAIN:
 	break;
       case GNUTLS_E_REHANDSHAKE:
 	do{
-	  ret = gnutls_handshake (session);
+	  ret = gnutls_handshake(session);
 	} while(ret == GNUTLS_E_AGAIN or ret == GNUTLS_E_INTERRUPTED);
-	if (ret < 0){
+	if(ret < 0){
 	  fprintf(stderr, "*** GnuTLS Re-handshake failed ***\n");
-	  gnutls_perror (ret);
+	  gnutls_perror(ret);
 	  retval = -1;
 	  goto mandos_end;
 	}
@@ -656,11 +666,11 @@
 	fprintf(stderr, "Unknown error while reading data from"
 		" encrypted session with Mandos server\n");
 	retval = -1;
-	gnutls_bye (session, GNUTLS_SHUT_RDWR);
+	gnutls_bye(session, GNUTLS_SHUT_RDWR);
 	goto mandos_end;
       }
     } else {
-      buffer_length += (size_t) ret;
+      buffer_length += (size_t) sret;
     }
   }
   
@@ -668,18 +678,18 @@
     fprintf(stderr, "Closing TLS session\n");
   }
   
-  gnutls_bye (session, GNUTLS_SHUT_RDWR);
+  gnutls_bye(session, GNUTLS_SHUT_RDWR);
   
-  if (buffer_length > 0){
+  if(buffer_length > 0){
     decrypted_buffer_size = pgp_packet_decrypt(mc, buffer,
 					       buffer_length,
 					       &decrypted_buffer);
-    if (decrypted_buffer_size >= 0){
+    if(decrypted_buffer_size >= 0){
       written = 0;
       while(written < (size_t) decrypted_buffer_size){
-	ret = (int)fwrite (decrypted_buffer + written, 1,
-			   (size_t)decrypted_buffer_size - written,
-			   stdout);
+	ret = (int)fwrite(decrypted_buffer + written, 1,
+			  (size_t)decrypted_buffer_size - written,
+			  stdout);
 	if(ret == 0 and ferror(stdout)){
 	  if(debug){
 	    fprintf(stderr, "Error writing encrypted data: %s\n",
@@ -702,11 +712,11 @@
   
  mandos_end:
   free(buffer);
-  ret = TEMP_FAILURE_RETRY(close(tcp_sd));
+  ret = (int)TEMP_FAILURE_RETRY(close(tcp_sd));
   if(ret == -1){
     perror("close");
   }
-  gnutls_deinit (session);
+  gnutls_deinit(session);
   return retval;
 }
 
@@ -730,7 +740,7 @@
   /* Called whenever a service has been resolved successfully or
      timed out */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_RESOLVER_FAILURE:
     fprintf(stderr, "(Avahi Resolver) Failed to resolve service '%s'"
@@ -744,11 +754,11 @@
       avahi_address_snprint(ip, sizeof(ip), address);
       if(debug){
 	fprintf(stderr, "Mandos server \"%s\" found on %s (%s, %"
-		PRIu16 ") on port %d\n", name, host_name, ip,
-		interface, port);
+		PRIdMAX ") on port %" PRIu16 "\n", name, host_name,
+		ip, (intmax_t)interface, port);
       }
       int ret = start_mandos_communication(ip, port, interface, mc);
-      if (ret == 0){
+      if(ret == 0){
 	avahi_simple_poll_quit(mc->simple_poll);
       }
     }
@@ -772,7 +782,7 @@
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
   
-  switch (event) {
+  switch(event) {
   default:
   case AVAHI_BROWSER_FAILURE:
     
@@ -787,7 +797,7 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if (!(avahi_s_service_resolver_new(mc->server, interface,
+    if(!(avahi_s_service_resolver_new(mc->server, interface,
 				       protocol, name, type, domain,
 				       AVAHI_PROTO_INET6, 0,
 				       resolve_callback, mc)))
@@ -811,6 +821,8 @@
     AvahiSServiceBrowser *sb = NULL;
     int error;
     int ret;
+    intmax_t tmpmax;
+    int numchars;
     int exitcode = EXIT_SUCCESS;
     const char *interface = "eth0";
     struct ifreq network;
@@ -862,11 +874,9 @@
 	{ .name = NULL }
       };
       
-      error_t parse_opt (int key, char *arg,
-			 struct argp_state *state) {
-	/* Get the INPUT argument from `argp_parse', which we know is
-	   a pointer to our plugin list pointer. */
-	switch (key) {
+      error_t parse_opt(int key, char *arg,
+			struct argp_state *state) {
+	switch(key) {
 	case 128:		/* --debug */
 	  debug = true;
 	  break;
@@ -883,18 +893,19 @@
 	  pubkey = arg;
 	  break;
 	case 129:		/* --dh-bits */
-	  errno = 0;
-	  mc.dh_bits = (unsigned int) strtol(arg, NULL, 10);
-	  if (errno){
-	    perror("strtol");
+	  ret = sscanf(arg, "%" SCNdMAX "%n", &tmpmax, &numchars);
+	  if(ret < 1 or tmpmax != (typeof(mc.dh_bits))tmpmax
+	     or arg[numchars] != '\0'){
+	    fprintf(stderr, "Bad number of DH bits\n");
 	    exit(EXIT_FAILURE);
 	  }
+	  mc.dh_bits = (typeof(mc.dh_bits))tmpmax;
 	  break;
 	case 130:		/* --priority */
 	  mc.priority = arg;
 	  break;
 	case ARGP_KEY_ARG:
-	  argp_usage (state);
+	  argp_usage(state);
 	case ARGP_KEY_END:
 	  break;
 	default:
@@ -907,8 +918,8 @@
 			   .args_doc = "",
 			   .doc = "Mandos client -- Get and decrypt"
 			   " passwords from a Mandos server" };
-      ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-      if (ret == ARGP_ERR_UNKNOWN){
+      ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+      if(ret == ARGP_ERR_UNKNOWN){
 	fprintf(stderr, "Unknown error while parsing arguments\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
@@ -939,7 +950,7 @@
 	  goto end;
 	}
       }
-      ret = TEMP_FAILURE_RETRY(close(sd));
+      ret = (int)TEMP_FAILURE_RETRY(close(sd));
       if(ret == -1){
 	perror("close");
       }
@@ -949,17 +960,17 @@
     gid = getgid();
     
     ret = setuid(uid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setuid");
     }
     
     setgid(gid);
-    if (ret == -1){
+    if(ret == -1){
       perror("setgid");
     }
     
     ret = init_gnutls_global(&mc, pubkey, seckey);
-    if (ret == -1){
+    if(ret == -1){
       fprintf(stderr, "init_gnutls_global failed\n");
       exitcode = EXIT_FAILURE;
       goto end;
@@ -996,13 +1007,15 @@
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
-      errno = 0;
-      uint16_t port = (uint16_t) strtol(address+1, NULL, 10);
-      if(errno){
-	perror("Bad port number");
+      uint16_t port;
+      ret = sscanf(address+1, "%" SCNdMAX "%n", &tmpmax, &numchars);
+      if(ret < 1 or tmpmax != (uint16_t)tmpmax
+	 or address[numchars+1] != '\0'){
+	fprintf(stderr, "Bad port number\n");
 	exitcode = EXIT_FAILURE;
 	goto end;
       }
+      port = (uint16_t)tmpmax;
       *address = '\0';
       address = connect_to;
       ret = start_mandos_communication(address, port, if_index, &mc);
@@ -1014,7 +1027,7 @@
       goto end;
     }
     
-    if (not debug){
+    if(not debug){
       avahi_set_log_function(empty_log);
     }
     
@@ -1023,7 +1036,7 @@
     
     /* Allocate main Avahi loop object */
     mc.simple_poll = avahi_simple_poll_new();
-    if (mc.simple_poll == NULL) {
+    if(mc.simple_poll == NULL) {
         fprintf(stderr, "Avahi: Failed to create simple poll"
 		" object.\n");
 	exitcode = EXIT_FAILURE;
@@ -1049,7 +1062,7 @@
     }
     
     /* Check if creating the Avahi server object succeeded */
-    if (mc.server == NULL) {
+    if(mc.server == NULL) {
         fprintf(stderr, "Failed to create Avahi server: %s\n",
 		avahi_strerror(error));
 	exitcode = EXIT_FAILURE;
@@ -1061,7 +1074,7 @@
 				     AVAHI_PROTO_INET6,
 				     "_mandos._tcp", NULL, 0,
 				     browse_callback, &mc);
-    if (sb == NULL) {
+    if(sb == NULL) {
         fprintf(stderr, "Failed to create service browser: %s\n",
 		avahi_strerror(avahi_server_errno(mc.server)));
 	exitcode = EXIT_FAILURE;
@@ -1070,7 +1083,7 @@
     
     /* Run the main loop */
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "Starting Avahi loop search\n");
     }
     
@@ -1078,23 +1091,23 @@
     
  end:
     
-    if (debug){
+    if(debug){
       fprintf(stderr, "%s exiting\n", argv[0]);
     }
     
     /* Cleanup things */
-    if (sb != NULL)
+    if(sb != NULL)
         avahi_s_service_browser_free(sb);
     
-    if (mc.server != NULL)
+    if(mc.server != NULL)
         avahi_server_free(mc.server);
     
-    if (mc.simple_poll != NULL)
+    if(mc.simple_poll != NULL)
         avahi_simple_poll_free(mc.simple_poll);
     
-    if (gnutls_initalized){
+    if(gnutls_initalized){
       gnutls_certificate_free_credentials(mc.cred);
-      gnutls_global_deinit ();
+      gnutls_global_deinit();
       gnutls_dh_params_deinit(mc.dh_params);
     }
     
@@ -1108,14 +1121,16 @@
       struct dirent *direntry;
       d = opendir(tempdir);
       if(d == NULL){
-	perror("opendir");
+	if(errno != ENOENT){
+	  perror("opendir");
+	}
       } else {
 	while(true){
 	  direntry = readdir(d);
 	  if(direntry == NULL){
 	    break;
 	  }
-	  if (direntry->d_type == DT_REG){
+	  if(direntry->d_type == DT_REG){
 	    char *fullname = NULL;
 	    ret = asprintf(&fullname, "%s/%s", tempdir,
 			   direntry->d_name);
@@ -1134,7 +1149,7 @@
 	closedir(d);
       }
       ret = rmdir(tempdir);
-      if(ret == -1){
+      if(ret == -1 and errno != ENOENT){
 	perror("rmdir");
       }
     }

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2008-10-04 01:55:56 +0000
+++ plugins.d/mandos-client.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2008-10-03">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2008-09-30 07:23:39 +0000
+++ plugins.d/password-prompt.c	2009-01-13 04:35:19 +0000
@@ -1,8 +1,9 @@
 /*  -*- coding: utf-8 -*- */
 /*
- * Passprompt - Read a password from the terminal and print it
- *
- * Copyright © 2008 Teddy Hogeborn & Björn Påhlsson
+ * Password-prompt - Read a password from the terminal and print it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -78,10 +79,8 @@
 	.doc = "Debug mode", .group = 3 },
       { .name = NULL }
     };
-  
+    
     error_t parse_opt (int key, char *arg, struct argp_state *state) {
-      /* Get the INPUT argument from `argp_parse', which we know is a
-	 pointer to our plugin list pointer. */
       switch (key) {
       case 'p':
 	prefix = arg;
@@ -90,7 +89,7 @@
 	debug = true;
 	break;
       case ARGP_KEY_ARG:
-	argp_usage (state);
+	argp_usage(state);
 	break;
       case ARGP_KEY_END:
 	break;
@@ -99,26 +98,26 @@
       }
       return 0;
     }
-  
+    
     struct argp argp = { .options = options, .parser = parse_opt,
 			 .args_doc = "",
 			 .doc = "Mandos password-prompt -- Read and"
 			 " output a password" };
-    ret = argp_parse (&argp, argc, argv, 0, 0, NULL);
-    if (ret == ARGP_ERR_UNKNOWN){
+    ret = argp_parse(&argp, argc, argv, 0, 0, NULL);
+    if(ret == ARGP_ERR_UNKNOWN){
       fprintf(stderr, "Unknown error while parsing arguments\n");
       return EXIT_FAILURE;
     }
   }
-    
-  if (debug){
+  
+  if(debug){
     fprintf(stderr, "Starting %s\n", argv[0]);
   }
-  if (debug){
+  if(debug){
     fprintf(stderr, "Storing current terminal attributes\n");
   }
   
-  if (tcgetattr(STDIN_FILENO, &t_old) != 0){
+  if(tcgetattr(STDIN_FILENO, &t_old) != 0){
     perror("tcgetattr");
     return EXIT_FAILURE;
   }
@@ -132,7 +131,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGINT, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -144,7 +143,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGHUP, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -156,7 +155,7 @@
     perror("sigaction");
     return EXIT_FAILURE;
   }
-  if (old_action.sa_handler != SIG_IGN){
+  if(old_action.sa_handler != SIG_IGN){
     ret = sigaction(SIGTERM, &new_action, NULL);
     if(ret == -1){
       perror("sigaction");
@@ -165,22 +164,22 @@
   }
   
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Removing echo flag from terminal attributes\n");
   }
   
   t_new = t_old;
   t_new.c_lflag &= ~ECHO;
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_new) != 0){
     perror("tcsetattr-echo");
     return EXIT_FAILURE;
   }
 
-  if (debug){
+  if(debug){
     fprintf(stderr, "Waiting for input from stdin \n");
   }
   while(true){
-    if (quit_now){
+    if(quit_now){
       if(debug){
 	fprintf(stderr, "Interrupted by signal, exiting.\n");
       }
@@ -212,7 +211,7 @@
       }
     }
     ret = getline(&buffer, &n, stdin);
-    if (ret > 0){
+    if(ret > 0){
       status = EXIT_SUCCESS;
       /* Make n = data size instead of allocated buffer size */
       n = (size_t)ret;
@@ -233,8 +232,8 @@
       }
       break;
     }
-    if (ret < 0){
-      if (errno != EINTR and not feof(stdin)){
+    if(ret < 0){
+      if(errno != EINTR and not feof(stdin)){
 	perror("getline");
 	status = EXIT_FAILURE;
 	break;
@@ -249,17 +248,17 @@
       fprintf(stderr, "getline() returned 0, retrying.\n");
     }
   }
-
+  
   free(buffer);
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "Restoring terminal attributes\n");
   }
-  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
+  if(tcsetattr(STDIN_FILENO, TCSAFLUSH, &t_old) != 0){
     perror("tcsetattr+echo");
   }
   
-  if (debug){
+  if(debug){
     fprintf(stderr, "%s is exiting with status %d\n", argv[0],
 	    status);
   }

=== modified file 'plugins.d/password-prompt.xml'
--- plugins.d/password-prompt.xml	2008-10-04 01:55:56 +0000
+++ plugins.d/password-prompt.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "password-prompt">
-<!ENTITY TIMESTAMP "2008-10-04">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/splashy.c'
--- plugins.d/splashy.c	2008-10-03 09:32:30 +0000
+++ plugins.d/splashy.c	2009-01-14 14:20:17 +0000
@@ -1,3 +1,28 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Splashy - Read a password from splashy and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
 #define _GNU_SOURCE		/* asprintf() */
 #include <signal.h>		/* sig_atomic_t, struct sigaction,
 				   sigemptyset(), sigaddset(), SIGINT,
@@ -5,12 +30,13 @@
 				   SIG_IGN, kill(), SIGKILL */
 #include <stddef.h>		/* NULL */
 #include <stdlib.h>		/* getenv() */
-#include <stdio.h>		/* asprintf(), perror() */
-#include <stdlib.h>		/* EXIT_FAILURE, free(), strtoul(),
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
+#include <stdlib.h>		/* EXIT_FAILURE, free(),
 				   EXIT_SUCCESS */
 #include <sys/types.h>		/* pid_t, DIR, struct dirent,
 				   ssize_t */
 #include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
 #include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
 #include <iso646.h>		/* not, or, and */
 #include <unistd.h>		/* readlink(), fork(), execl(),
@@ -72,10 +98,18 @@
     for(struct dirent *proc_ent = readdir(proc_dir);
 	proc_ent != NULL;
 	proc_ent = readdir(proc_dir)){
-      pid_t pid = (pid_t) strtoul(proc_ent->d_name, NULL, 10);
-      if(pid == 0){
-	/* Not a process */
-	continue;
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
       }
       /* Find the executable name by doing readlink() on the
 	 /proc/<pid>/exe link */
@@ -95,6 +129,10 @@
 	struct stat exe_stat;
 	ret = lstat(exe_link, &exe_stat);
 	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
 	  perror("lstat");
 	  free(exe_link);
 	  free(prompt);
@@ -238,7 +276,7 @@
     /* Child; will become new splashy process */
     
     /* Make the effective user ID (root) the only user ID instead of
-       the real user ID (mandos) */
+       the real user ID (_mandos) */
     ret = setuid(geteuid());
     if(ret == -1){
       perror("setuid");

=== modified file 'plugins.d/splashy.xml'
--- plugins.d/splashy.xml	2008-10-04 03:11:39 +0000
+++ plugins.d/splashy.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "splashy">
-<!ENTITY TIMESTAMP "2008-10-04">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

=== modified file 'plugins.d/usplash.c'
--- plugins.d/usplash.c	2008-10-03 09:32:30 +0000
+++ plugins.d/usplash.c	2009-01-14 14:20:17 +0000
@@ -1,3 +1,28 @@
+/*  -*- coding: utf-8 -*- */
+/*
+ * Usplash - Read a password from usplash and output it
+ * 
+ * Copyright © 2008,2009 Teddy Hogeborn
+ * Copyright © 2008,2009 Björn Påhlsson
+ * 
+ * This program is free software: you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ * 
+ * Contact the authors at <https://www.fukt.bsnet.se/~belorn/> and
+ * <https://www.fukt.bsnet.se/~teddy/>.
+ */
+
 #define _GNU_SOURCE		/* asprintf() */
 #include <signal.h>		/* sig_atomic_t, struct sigaction,
 				   sigemptyset(), sigaddset(), SIGINT,
@@ -11,17 +36,17 @@
 				   dirent */
 #include <stddef.h>		/* NULL */
 #include <string.h>		/* strlen(), memcmp() */
-#include <stdio.h>		/* asprintf(), perror() */
+#include <stdio.h>		/* asprintf(), perror(), sscanf() */
 #include <unistd.h>		/* close(), write(), readlink(),
 				   read(), STDOUT_FILENO, sleep(),
 				   fork(), setuid(), geteuid(),
 				   setsid(), chdir(), dup2(),
 				   STDERR_FILENO, execv() */
-#include <stdlib.h>		/* free(), EXIT_FAILURE, strtoul(),
-				   realloc(), EXIT_SUCCESS, malloc(),
-				   _exit() */
+#include <stdlib.h>		/* free(), EXIT_FAILURE, realloc(),
+				   EXIT_SUCCESS, malloc(), _exit() */
 #include <stdlib.h>		/* getenv() */
 #include <dirent.h>		/* opendir(), readdir(), closedir() */
+#include <inttypes.h>		/* intmax_t, SCNdMAX */
 #include <sys/stat.h>		/* struct stat, lstat(), S_ISLNK */
 
 sig_atomic_t interrupted_by_signal = 0;
@@ -67,10 +92,11 @@
   }
   
   size_t written = 0;
+  ssize_t sret = 0;
   while(not interrupted_by_signal and written < cmd_line_len){
-    ret = write(fifo_fd, cmd_line + written,
-		cmd_line_len - written);
-    if(ret == -1){
+    sret = write(fifo_fd, cmd_line + written,
+		 cmd_line_len - written);
+    if(sret == -1){
       if(errno != EINTR or interrupted_by_signal){
 	int e = errno;
 	close(fifo_fd);
@@ -81,7 +107,7 @@
 	continue;
       }
     }
-    written += (size_t)ret;
+    written += (size_t)sret;
   }
   free(cmd_line_alloc);
   do{
@@ -144,10 +170,18 @@
     for(struct dirent *proc_ent = readdir(proc_dir);
 	proc_ent != NULL;
 	proc_ent = readdir(proc_dir)){
-      pid_t pid = (pid_t) strtoul(proc_ent->d_name, NULL, 10);
-      if(pid == 0){
-	/* Not a process */
-	continue;
+      pid_t pid;
+      {
+	intmax_t tmpmax;
+	int numchars;
+	ret = sscanf(proc_ent->d_name, "%" SCNdMAX "%n", &tmpmax,
+		     &numchars);
+	if(ret < 1 or tmpmax != (pid_t)tmpmax
+	   or proc_ent->d_name[numchars] != '\0'){
+	  /* Not a process */
+	  continue;
+	}
+	pid = (pid_t)tmpmax;
       }
       /* Find the executable name by doing readlink() on the
 	 /proc/<pid>/exe link */
@@ -167,6 +201,10 @@
 	struct stat exe_stat;
 	ret = lstat(exe_link, &exe_stat);
 	if(ret == -1){
+	  if(errno == ENOENT){
+	    free(exe_link);
+	    continue;
+	  }
 	  perror("lstat");
 	  free(exe_link);
 	  free(prompt);
@@ -182,9 +220,6 @@
 	
 	sret = readlink(exe_link, exe_target, sizeof(exe_target));
 	free(exe_link);
-	if(sret == -1){
-	  continue;
-	}
       }
       if((sret == ((ssize_t)sizeof(exe_target)-1))
 	 and (memcmp(usplash_name, exe_target,
@@ -459,7 +494,7 @@
     /* Child; will become new usplash process */
     
     /* Make the effective user ID (root) the only user ID instead of
-       the real user ID (mandos) */
+       the real user ID (_mandos) */
     ret = setuid(geteuid());
     if(ret == -1){
       perror("setuid");

=== modified file 'plugins.d/usplash.xml'
--- plugins.d/usplash.xml	2008-10-04 03:11:39 +0000
+++ plugins.d/usplash.xml	2009-01-04 21:54:55 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "usplash">
-<!ENTITY TIMESTAMP "2008-10-04">
+<!ENTITY TIMESTAMP "2009-01-04">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -32,6 +32,7 @@
     </authorgroup>
     <copyright>
       <year>2008</year>
+      <year>2009</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>

